@@ -2,7 +2,7 @@
    File highlight plugin.
    Reading and parse rules from ini-files
 
-   Copyright (C) 2009-2015
+   Copyright (C) 2009-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -38,8 +38,6 @@
 
 /*** global variables ****************************************************************************/
 
-extern mc_skin_t mc_skin__default;
-
 /*** file scope macro definitions ****************************************************************/
 
 /*** file scope type declarations ****************************************************************/
@@ -115,7 +113,7 @@ mc_fhl_parse_get_regexp (mc_fhl_t * fhl, const gchar * group_name)
 
     mc_filter = g_new0 (mc_fhl_filter_t, 1);
     mc_filter->type = MC_FLHGH_T_FREGEXP;
-    mc_filter->search_condition = mc_search_new (regexp, -1, DEFAULT_CHARSET);
+    mc_filter->search_condition = mc_search_new (regexp, DEFAULT_CHARSET);
     mc_filter->search_condition->is_case_sensitive = TRUE;
     mc_filter->search_condition->search_type = MC_SEARCH_T_REGEX;
 
@@ -160,9 +158,9 @@ mc_fhl_parse_get_extensions (mc_fhl_t * fhl, const gchar * group_name)
 
     mc_filter = g_new0 (mc_fhl_filter_t, 1);
     mc_filter->type = MC_FLHGH_T_FREGEXP;
-    mc_filter->search_condition = mc_search_new (buf->str, buf->len, DEFAULT_CHARSET);
+    mc_filter->search_condition = mc_search_new_len (buf->str, buf->len, DEFAULT_CHARSET);
     mc_filter->search_condition->is_case_sensitive =
-        mc_config_get_bool (fhl->config, group_name, "extensions_case", TRUE);
+        mc_config_get_bool (fhl->config, group_name, "extensions_case", FALSE);
     mc_filter->search_condition->search_type = MC_SEARCH_T_REGEX;
 
     mc_fhl_parse_fill_color_info (mc_filter, fhl, group_name);
