@@ -1,7 +1,7 @@
 /*
    Interface to the terminal controlling library.
 
-   Copyright (C) 2005-2015
+   Copyright (C) 2005-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -50,11 +50,6 @@
 
 /*** global variables ****************************************************************************/
 
-/* If true program softkeys (HP terminals only) on startup and after every
-   command ran in the subshell to the description found in the termcap/terminfo
-   database */
-int reset_hp_softkeys = 0;
-
 int mc_tty_frm[MC_TTY_FRM_MAX];
 
 /*** file scope macro definitions ****************************************************************/
