@@ -1,7 +1,7 @@
 /*
    Various utilities - Unix variants
 
-   Copyright (C) 1994-2015
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -44,7 +44,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <fcntl.h>
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
@@ -121,6 +120,7 @@ static int_cache gid_cache[GID_CACHE_SIZE];
 static int error_pipe[2];       /* File descriptors of error pipe */
 static int old_error;           /* File descriptor of old standard error */
 
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
@@ -216,8 +216,8 @@ my_system_make_arg_array (int flags, const char *shell, char **execute_name)
 
     if ((flags & EXECUTE_AS_SHELL) != 0)
     {
-        g_ptr_array_add (args_array, g_strdup (shell));
-        g_ptr_array_add (args_array, g_strdup ("-c"));
+        g_ptr_array_add (args_array, (gpointer) shell);
+        g_ptr_array_add (args_array, (gpointer) "-c");
         *execute_name = g_strdup (shell);
     }
     else
@@ -230,7 +230,7 @@ my_system_make_arg_array (int flags, const char *shell, char **execute_name)
         else
             *execute_name = g_strndup (shell, (gsize) (shell_token - shell));
 
-        g_ptr_array_add (args_array, g_strdup (shell));
+        g_ptr_array_add (args_array, (gpointer) shell);
     }
     return args_array;
 }
@@ -283,56 +283,56 @@ mc_pread_stream (mc_pipe_stream_t * ps, const fd_set * fds)
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
-char *
-get_owner (int uid)
+const char *
+get_owner (uid_t uid)
 {
     struct passwd *pwd;
     char *name;
-    static int uid_last;
+    static uid_t uid_last;
 
-    name = i_cache_match (uid, uid_cache, UID_CACHE_SIZE);
+    name = i_cache_match ((int) uid, uid_cache, UID_CACHE_SIZE);
     if (name != NULL)
         return name;
 
     pwd = getpwuid (uid);
     if (pwd != NULL)
     {
-        i_cache_add (uid, uid_cache, UID_CACHE_SIZE, pwd->pw_name, &uid_last);
+        i_cache_add ((int) uid, uid_cache, UID_CACHE_SIZE, pwd->pw_name, (int *) &uid_last);
         return pwd->pw_name;
     }
     else
     {
         static char ibuf[10];
 
-        g_snprintf (ibuf, sizeof (ibuf), "%d", uid);
+        g_snprintf (ibuf, sizeof (ibuf), "%d", (int) uid);
         return ibuf;
     }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-char *
-get_group (int gid)
+const char *
+get_group (gid_t gid)
 {
     struct group *grp;
     char *name;
-    static int gid_last;
+    static gid_t gid_last;
 
-    name = i_cache_match (gid, gid_cache, GID_CACHE_SIZE);
+    name = i_cache_match ((int) gid, gid_cache, GID_CACHE_SIZE);
     if (name != NULL)
         return name;
 
     grp = getgrgid (gid);
     if (grp != NULL)
     {
-        i_cache_add (gid, gid_cache, GID_CACHE_SIZE, grp->gr_name, &gid_last);
+        i_cache_add ((int) gid, gid_cache, GID_CACHE_SIZE, grp->gr_name, (int *) &gid_last);
         return grp->gr_name;
     }
     else
     {
         static char gbuf[10];
 
-        g_snprintf (gbuf, sizeof (gbuf), "%d", gid);
+        g_snprintf (gbuf, sizeof (gbuf), "%d", (int) gid);
         return gbuf;
     }
 }
@@ -358,6 +358,7 @@ save_stop_handler (void)
  */
 
 void
+/* __attribute__ ((noreturn)) */
 my_exit (int status)
 {
     _exit (status);
@@ -395,7 +396,6 @@ my_system (int flags, const char *shell, const char *command)
  * @return 0 if successfull, -1 otherwise
  */
 
-
 int
 my_systeml (int flags, const char *shell, ...)
 {
@@ -454,7 +454,8 @@ my_systemv (const char *command, char *const argv[])
             execvp (command, argv);
             my_exit (127);      /* Exec error */
         }
-        break;
+        MC_FALLTHROUGH;
+        /* no break here, or unreachable-code warning by no returning my_exit() */
     default:
         status = 0;
         break;
@@ -511,7 +512,7 @@ mc_pipe_t *
 mc_popen (const char *command, GError ** error)
 {
     mc_pipe_t *p;
-    char **argv;
+    const char *const argv[] = { "/bin/sh", "sh", "-c", command, NULL };
 
     p = g_try_new (mc_pipe_t, 1);
     if (p == NULL)
@@ -521,23 +522,15 @@ mc_popen (const char *command, GError ** error)
         goto ret_err;
     }
 
-    if (!g_shell_parse_argv (command, NULL, &argv, error))
-    {
-        mc_replace_error (error, MC_PIPE_ERROR_PARSE_COMMAND, "%s",
-                          _("Cannot parse command for pipe"));
-        goto ret_err;
-    }
-
-    if (!g_spawn_async_with_pipes (NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
-                                   &p->child_pid, NULL, &p->out.fd, &p->err.fd, error))
+    if (!g_spawn_async_with_pipes
+        (NULL, (gchar **) argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_FILE_AND_ARGV_ZERO,
+         NULL, NULL, &p->child_pid, NULL, &p->out.fd, &p->err.fd, error))
     {
         mc_replace_error (error, MC_PIPE_ERROR_CREATE_PIPE_STREAM, "%s",
                           _("Cannot create pipe streams"));
         goto ret_err;
     }
 
-    g_strfreev (argv);
-
     p->out.buf[0] = '\0';
     p->out.len = MC_PIPE_BUFSIZE;
     p->out.null_term = FALSE;
@@ -604,7 +597,7 @@ mc_pread (mc_pipe_t * p, GError ** error)
     if (read_err)
     {
         FD_SET (p->err.fd, &fds);
-        maxfd = max (maxfd, p->err.fd);
+        maxfd = MAX (maxfd, p->err.fd);
     }
 
     /* no timeout */
@@ -692,10 +685,8 @@ tilde_expand (const char *directory)
     else
     {
         q = strchr (p, PATH_SEP);
-        if (!q)
-        {
+        if (q == NULL)
             passwd = getpwnam (p);
-        }
         else
         {
             char *name;
@@ -708,7 +699,7 @@ tilde_expand (const char *directory)
     }
 
     /* If we can't figure the user name, leave tilde unexpanded */
-    if (!passwd)
+    if (passwd == NULL)
         return g_strdup (directory);
 
     return g_strconcat (passwd->pw_dir, PATH_SEP_STR, q, (char *) NULL);
@@ -725,9 +716,8 @@ void
 open_error_pipe (void)
 {
     if (pipe (error_pipe) < 0)
-    {
         message (D_NORMAL, _("Warning"), _("Pipe failed"));
-    }
+
     old_error = dup (STDERR_FILENO);
     if (old_error < 0 || close (STDERR_FILENO) != 0 || dup (error_pipe[1]) != STDERR_FILENO)
     {
@@ -747,10 +737,12 @@ open_error_pipe (void)
          * of following hack.
          */
         int fd_flags;
+
         fd_flags = fcntl (error_pipe[0], F_GETFL, NULL);
         if (fd_flags != -1)
         {
             fd_flags |= O_NONBLOCK;
+
             if (fcntl (error_pipe[0], F_SETFL, fd_flags) == -1)
             {
                 /* TODO: handle it somehow */
@@ -798,7 +790,7 @@ close_error_pipe (int error, const char *text)
             return 1;
         }
         close (old_error);
-        len = read (error_pipe[0], msg, MAX_PIPE_SIZE - 1);
+        len = read (error_pipe[0], msg, sizeof (msg) - 1);
 
         if (len >= 0)
             msg[len] = 0;
@@ -845,22 +837,19 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
     /* Detect and preserve UNC paths: //server/... */
     if ((flags & CANON_PATH_GUARDUNC) != 0 && IS_PATH_SEP (path[0]) && IS_PATH_SEP (path[1]))
     {
-        p = path + 2;
-        while (p[0] != '\0' && !IS_PATH_SEP (p[0]))
-            p++;
+        for (p = path + 2; p[0] != '\0' && !IS_PATH_SEP (p[0]); p++)
+            ;
         if (IS_PATH_SEP (p[0]) && p > path + 2)
             lpath = p;
     }
 
-    if (!lpath[0] || !lpath[1])
+    if (lpath[0] == '\0' || lpath[1] == '\0')
         return;
 
-    if (flags & CANON_PATH_JOINSLASHES)
+    if ((flags & CANON_PATH_JOINSLASHES) != 0)
     {
         /* Collapse multiple slashes */
-        p = lpath;
-        while (*p)
-        {
+        for (p = lpath; *p != '\0'; p++)
             if (IS_PATH_SEP (p[0]) && IS_PATH_SEP (p[1]) && (p == lpath || *(p - 1) != ':'))
             {
                 s = p + 1;
@@ -868,88 +857,70 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
                     ;
                 str_move (p + 1, s);
             }
-            p++;
-        }
-    }
 
-    if (flags & CANON_PATH_JOINSLASHES)
-    {
         /* Collapse "/./" -> "/" */
-        p = lpath;
-        while (*p)
-        {
+        for (p = lpath; *p != '\0';)
             if (IS_PATH_SEP (p[0]) && p[1] == '.' && IS_PATH_SEP (p[2]))
                 str_move (p, p + 2);
             else
                 p++;
-        }
     }
 
-    if (flags & CANON_PATH_REMSLASHDOTS)
+    if ((flags & CANON_PATH_REMSLASHDOTS) != 0)
     {
         size_t len;
 
         /* Remove trailing slashes */
-        p = lpath + strlen (lpath) - 1;
-        while (p > lpath && IS_PATH_SEP (*p))
+        for (p = lpath + strlen (lpath) - 1; p > lpath && IS_PATH_SEP (*p); p--)
         {
-            if (p >= lpath - (url_delim_len + 1)
+            if (p >= lpath + url_delim_len - 1
                 && strncmp (p - url_delim_len + 1, VFS_PATH_URL_DELIMITER, url_delim_len) == 0)
                 break;
-            *p-- = 0;
+            *p = '\0';
         }
 
         /* Remove leading "./" */
         if (lpath[0] == '.' && IS_PATH_SEP (lpath[1]))
         {
-            if (lpath[2] == 0)
+            if (lpath[2] == '\0')
             {
-                lpath[1] = 0;
+                lpath[1] = '\0';
                 return;
             }
-            else
-            {
-                str_move (lpath, lpath + 2);
-            }
+
+            str_move (lpath, lpath + 2);
         }
 
         /* Remove trailing "/" or "/." */
         len = strlen (lpath);
         if (len < 2)
             return;
+
         if (IS_PATH_SEP (lpath[len - 1])
             && (len < url_delim_len
                 || strncmp (lpath + len - url_delim_len, VFS_PATH_URL_DELIMITER,
                             url_delim_len) != 0))
-        {
             lpath[len - 1] = '\0';
-        }
-        else
+        else if (lpath[len - 1] == '.' && IS_PATH_SEP (lpath[len - 2]))
         {
-            if (lpath[len - 1] == '.' && IS_PATH_SEP (lpath[len - 2]))
+            if (len == 2)
             {
-                if (len == 2)
-                {
-                    lpath[1] = '\0';
-                    return;
-                }
-                else
-                {
-                    lpath[len - 2] = '\0';
-                }
+                lpath[1] = '\0';
+                return;
             }
+
+            lpath[len - 2] = '\0';
         }
     }
 
-    if (flags & CANON_PATH_REMDOUBLEDOTS)
+    /* Collapse "/.." with the previous part of path */
+    if ((flags & CANON_PATH_REMDOUBLEDOTS) != 0)
     {
 #ifdef HAVE_CHARSET
         const size_t enc_prefix_len = strlen (VFS_ENCODING_PREFIX);
 #endif /* HAVE_CHARSET */
 
-        /* Collapse "/.." with the previous part of path */
-        p = lpath;
-        while (p[0] && p[1] && p[2])
+        for (p = lpath; p[0] != '\0' && p[1] != '\0' && p[2] != '\0';)
         {
             if (!IS_PATH_SEP (p[0]) || p[1] != '.' || p[2] != '.'
                 || (!IS_PATH_SEP (p[3]) && p[3] != '\0'))
@@ -974,7 +945,7 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
                     && strncmp (s - url_delim_len, VFS_PATH_URL_DELIMITER, url_delim_len) == 0)
                 {
                     char *vfs_prefix = s - url_delim_len;
-                    struct vfs_class *vclass;
+                    vfs_class *vclass;
 
                     while (vfs_prefix > lpath && !IS_PATH_SEP (*--vfs_prefix))
                         ;
@@ -985,14 +956,10 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
                     vclass = vfs_prefix_to_class (vfs_prefix);
                     *(s - url_delim_len) = *VFS_PATH_URL_DELIMITER;
 
-                    if (vclass != NULL)
+                    if (vclass != NULL && (vclass->flags & VFS_REMOTE) != 0)
                     {
-                        struct vfs_s_subclass *sub = (struct vfs_s_subclass *) vclass->data;
-                        if (sub != NULL && sub->flags & VFS_S_REMOTE)
-                        {
-                            s = vfs_prefix;
-                            continue;
-                        }
+                        s = vfs_prefix;
+                        continue;
                     }
                 }
 
@@ -1011,7 +978,7 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
                 continue;
             }
 
-            if (p[3] != 0)
+            if (p[3] != '\0')
             {
                 if (s == lpath && IS_PATH_SEP (*s))
                 {
@@ -1030,7 +997,8 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
 #endif /* HAVE_CHARSET */
                         str_move (s, p + 4);
                 }
-                p = (s > lpath) ? s - 1 : s;
+
+                p = s > lpath ? s - 1 : s;
                 continue;
             }
 
@@ -1058,9 +1026,8 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
 
                     /* search for the previous token */
                     /* IS_PATH_SEP (s[-1]) */
-                    p = s - 1;
-                    while (p >= lpath && !IS_PATH_SEP (*p))
-                        p--;
+                    for (p = s - 1; p >= lpath && !IS_PATH_SEP (*p); p--)
+                        ;
 
                     if (p >= lpath)
                         continue;
@@ -1095,154 +1062,186 @@ canonicalize_pathname (char *path)
 int
 gettimeofday (struct timeval *tp, void *tzp)
 {
+    (void) tzp;
+
     return get_process_stats (tp, PS_SELF, 0, 0);
 }
 #endif /* HAVE_GET_PROCESS_STATS */
 
 /* --------------------------------------------------------------------------------------------- */
 
-#ifndef HAVE_REALPATH
 char *
 mc_realpath (const char *path, char *resolved_path)
 {
-    char copy_path[PATH_MAX];
-    char link_path[PATH_MAX];
-    char got_path[PATH_MAX];
-    char *new_path = got_path;
-    char *max_path;
-    int readlinks = 0;
-    int n;
+#ifdef HAVE_CHARSET
+    const char *p = path;
+    gboolean absolute_path = FALSE;
 
-    /* Make a copy of the source path since we may need to modify it. */
-    if (strlen (path) >= PATH_MAX - 2)
+    if (IS_PATH_SEP (*p))
     {
-        errno = ENAMETOOLONG;
-        return NULL;
+        absolute_path = TRUE;
+        p++;
     }
-    strcpy (copy_path, path);
-    path = copy_path;
-    max_path = copy_path + PATH_MAX - 2;
-    /* If it's a relative pathname use getwd for starters. */
-    if (!IS_PATH_SEP (*path))
+
+    /* ignore encoding: skip "#enc:" */
+    if (g_str_has_prefix (p, VFS_ENCODING_PREFIX))
     {
-        new_path = g_get_current_dir ();
-        if (new_path == NULL)
+        p += strlen (VFS_ENCODING_PREFIX);
+        p = strchr (p, PATH_SEP);
+        if (p != NULL)
         {
-            strcpy (got_path, "");
+            if (!absolute_path && p[1] != '\0')
+                p++;
+
+            path = p;
+        }
+    }
+#endif /* HAVE_CHARSET */
+
+#ifdef HAVE_REALPATH
+    return realpath (path, resolved_path);
+#else
+    {
+        char copy_path[PATH_MAX];
+        char got_path[PATH_MAX];
+        char *new_path = got_path;
+        char *max_path;
+#ifdef S_IFLNK
+        char link_path[PATH_MAX];
+        int readlinks = 0;
+        int n;
+#endif /* S_IFLNK */
+
+        /* Make a copy of the source path since we may need to modify it. */
+        if (strlen (path) >= PATH_MAX - 2)
+        {
+            errno = ENAMETOOLONG;
+            return NULL;
+        }
+
+        strcpy (copy_path, path);
+        path = copy_path;
+        max_path = copy_path + PATH_MAX - 2;
+        /* If it's a relative pathname use getwd for starters. */
+        if (!IS_PATH_SEP (*path))
+        {
+            new_path = g_get_current_dir ();
+            if (new_path == NULL)
+                strcpy (got_path, "");
+            else
+            {
+                g_snprintf (got_path, sizeof (got_path), "%s", new_path);
+                g_free (new_path);
+                new_path = got_path;
+            }
+
+            new_path += strlen (got_path);
+            if (!IS_PATH_SEP (new_path[-1]))
+                *new_path++ = PATH_SEP;
         }
         else
         {
-            g_snprintf (got_path, PATH_MAX, "%s", new_path);
-            g_free (new_path);
-            new_path = got_path;
-        }
-
-        new_path += strlen (got_path);
-        if (!IS_PATH_SEP (new_path[-1]))
             *new_path++ = PATH_SEP;
-    }
-    else
-    {
-        *new_path++ = PATH_SEP;
-        path++;
-    }
-    /* Expand each slash-separated pathname component. */
-    while (*path != '\0')
-    {
-        /* Ignore stray "/". */
-        if (IS_PATH_SEP (*path))
-        {
             path++;
-            continue;
         }
-        if (*path == '.')
+        /* Expand each slash-separated pathname component. */
+        while (*path != '\0')
         {
-            /* Ignore ".". */
-            if (path[1] == '\0' || IS_PATH_SEP (path[1]))
+            /* Ignore stray "/". */
+            if (IS_PATH_SEP (*path))
             {
                 path++;
                 continue;
             }
-            if (path[1] == '.')
+            if (*path == '.')
             {
-                if (path[2] == '\0' || IS_PATH_SEP (path[2]))
+                /* Ignore ".". */
+                if (path[1] == '\0' || IS_PATH_SEP (path[1]))
                 {
-                    path += 2;
-                    /* Ignore ".." at root. */
-                    if (new_path == got_path + 1)
-                        continue;
-                    /* Handle ".." by backing up. */
-                    while (!IS_PATH_SEP ((--new_path)[-1]))
-                        ;
+                    path++;
                     continue;
                 }
+                if (path[1] == '.')
+                {
+                    if (path[2] == '\0' || IS_PATH_SEP (path[2]))
+                    {
+                        path += 2;
+                        /* Ignore ".." at root. */
+                        if (new_path == got_path + 1)
+                            continue;
+                        /* Handle ".." by backing up. */
+                        while (!IS_PATH_SEP ((--new_path)[-1]))
+                            ;
+                        continue;
+                    }
+                }
             }
-        }
-        /* Safely copy the next pathname component. */
-        while (*path != '\0' && !IS_PATH_SEP (*path))
-        {
-            if (path > max_path)
+            /* Safely copy the next pathname component. */
+            while (*path != '\0' && !IS_PATH_SEP (*path))
             {
-                errno = ENAMETOOLONG;
-                return NULL;
+                if (path > max_path)
+                {
+                    errno = ENAMETOOLONG;
+                    return NULL;
+                }
+                *new_path++ = *path++;
             }
-            *new_path++ = *path++;
-        }
 #ifdef S_IFLNK
-        /* Protect against infinite loops. */
-        if (readlinks++ > MAXSYMLINKS)
-        {
-            errno = ELOOP;
-            return NULL;
-        }
-        /* See if latest pathname component is a symlink. */
-        *new_path = '\0';
-        n = readlink (got_path, link_path, PATH_MAX - 1);
-        if (n < 0)
-        {
-            /* EINVAL means the file exists but isn't a symlink. */
-            if (errno != EINVAL)
+            /* Protect against infinite loops. */
+            if (readlinks++ > MAXSYMLINKS)
             {
-                /* Make sure it's null terminated. */
-                *new_path = '\0';
-                strcpy (resolved_path, got_path);
+                errno = ELOOP;
                 return NULL;
             }
-        }
-        else
-        {
-            /* Note: readlink doesn't add the null byte. */
-            link_path[n] = '\0';
-            if (IS_PATH_SEP (*link_path))
-                /* Start over for an absolute symlink. */
-                new_path = got_path;
+            /* See if latest pathname component is a symlink. */
+            *new_path = '\0';
+            n = readlink (got_path, link_path, PATH_MAX - 1);
+            if (n < 0)
+            {
+                /* EINVAL means the file exists but isn't a symlink. */
+                if (errno != EINVAL)
+                {
+                    /* Make sure it's null terminated. */
+                    *new_path = '\0';
+                    strcpy (resolved_path, got_path);
+                    return NULL;
+                }
+            }
             else
-                /* Otherwise back up over this component. */
-                while (!IS_PATH_SEP (*(--new_path)))
-                    ;
-            /* Safe sex check. */
-            if (strlen (path) + n >= PATH_MAX - 2)
             {
-                errno = ENAMETOOLONG;
-                return NULL;
+                /* Note: readlink doesn't add the null byte. */
+                link_path[n] = '\0';
+                if (IS_PATH_SEP (*link_path))
+                    /* Start over for an absolute symlink. */
+                    new_path = got_path;
+                else
+                    /* Otherwise back up over this component. */
+                    while (!IS_PATH_SEP (*(--new_path)))
+                        ;
+                /* Safe sex check. */
+                if (strlen (path) + n >= PATH_MAX - 2)
+                {
+                    errno = ENAMETOOLONG;
+                    return NULL;
+                }
+                /* Insert symlink contents into path. */
+                strcat (link_path, path);
+                strcpy (copy_path, link_path);
+                path = copy_path;
             }
-            /* Insert symlink contents into path. */
-            strcat (link_path, path);
-            strcpy (copy_path, link_path);
-            path = copy_path;
-        }
 #endif /* S_IFLNK */
-        *new_path++ = PATH_SEP;
+            *new_path++ = PATH_SEP;
+        }
+        /* Delete trailing slash but don't whomp a lone slash. */
+        if (new_path != got_path + 1 && IS_PATH_SEP (new_path[-1]))
+            new_path--;
+        /* Make sure it's null terminated. */
+        *new_path = '\0';
+        strcpy (resolved_path, got_path);
+        return resolved_path;
     }
-    /* Delete trailing slash but don't whomp a lone slash. */
-    if (new_path != got_path + 1 && IS_PATH_SEP (new_path[-1]))
-        new_path--;
-    /* Make sure it's null terminated. */
-    *new_path = '\0';
-    strcpy (resolved_path, got_path);
-    return resolved_path;
-}
 #endif /* HAVE_REALPATH */
+}
 
 /* --------------------------------------------------------------------------------------------- */
 /**
@@ -1289,10 +1288,8 @@ get_user_permissions (struct stat *st)
         return 0;
 
     for (i = 0; i < ngroups; i++)
-    {
         if (st->st_gid == groups[i])
             return 1;
-    }
 
     return 2;
 }
