@@ -1,7 +1,7 @@
 /*
    Virtual File System path handlers
 
-   Copyright (C) 2011-2015
+   Copyright (C) 2011-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -135,13 +135,16 @@ _vfs_split_with_semi_skip_count (char *path, const char **inpath, const char **o
 static char *
 vfs_canon (const char *path)
 {
+    char *result;
+
     if (path == NULL)
         vfs_die ("Cannot canonicalize NULL");
 
-    /* Relative to current directory */
     if (!IS_PATH_SEP (*path))
     {
-        char *result, *local;
+        /* Relative to current directory */
+
+        char *local;
 
         if (g_str_has_prefix (path, VFS_ENCODING_PREFIX))
         {
@@ -149,32 +152,27 @@ vfs_canon (const char *path)
                encoding prefix placed at start of string without the leading slash
                should be autofixed by adding the leading slash
              */
-            local = mc_build_filename (PATH_SEP_STR, path, NULL);
+            local = mc_build_filename (PATH_SEP_STR, path, (char *) NULL);
         }
         else
         {
-            char *curr_dir;
+            const char *curr_dir;
 
             curr_dir = vfs_get_current_dir ();
-            local = mc_build_filename (curr_dir, path, NULL);
-            g_free (curr_dir);
+            local = mc_build_filename (curr_dir, path, (char *) NULL);
         }
         result = vfs_canon (local);
         g_free (local);
-        return result;
     }
-
-    /*
-     * So we have path of following form:
-     * /p1/p2#op/.././././p3#op/p4. Good luck.
-     */
+    else
     {
-        char *result;
+        /* Absolute path */
 
         result = g_strdup (path);
         canonicalize_pathname (result);
-        return result;
     }
+
+    return result;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -238,13 +236,11 @@ static void
 vfs_path_url_split (vfs_path_element_t * path_element, const char *path)
 {
     char *pcopy;
-    const char *pend;
     char *colon, *at, *rest;
 
     path_element->port = 0;
 
     pcopy = g_strdup (path);
-    pend = pcopy + strlen (pcopy);
 
     /* search for any possible user */
     at = strrchr (pcopy, '@');
@@ -254,9 +250,12 @@ vfs_path_url_split (vfs_path_element_t * path_element, const char *path)
         rest = pcopy;
     else
     {
+        const char *pend;
         char *inner_colon;
 
+        pend = strchr (at, '\0');
         *at = '\0';
+
         inner_colon = strchr (pcopy, ':');
         if (inner_colon != NULL)
         {
@@ -282,9 +281,9 @@ vfs_path_url_split (vfs_path_element_t * path_element, const char *path)
         colon = strchr (++rest, ']');
         if (colon != NULL)
         {
-            colon[0] = '\0';
-            colon[1] = '\0';
+            *colon = '\0';
             colon++;
+            *colon = '\0';
             path_element->ipv6 = TRUE;
         }
     }
@@ -309,6 +308,8 @@ vfs_path_url_split (vfs_path_element_t * path_element, const char *path)
                 case 'r':
                     path_element->port = 2;
                     break;
+                default:
+                    break;
                 }
             }
     }
@@ -335,7 +336,7 @@ vfs_get_class_by_name (const char *class_name)
 
     for (i = 0; i < vfs__classes_list->len; i++)
     {
-        struct vfs_class *vfs = (struct vfs_class *) g_ptr_array_index (vfs__classes_list, i);
+        struct vfs_class *vfs = VFS_CLASS (g_ptr_array_index (vfs__classes_list, i));
         if ((vfs->name != NULL) && (strcmp (vfs->name, class_name) == 0))
             return vfs;
     }
@@ -367,14 +368,13 @@ vfs_path_is_str_path_deprecated (const char *path_str)
 */
 
 static vfs_path_t *
-vfs_path_from_str_deprecated_parser (char *path, vfs_path_flag_t flags)
+vfs_path_from_str_deprecated_parser (char *path)
 {
     vfs_path_t *vpath;
     vfs_path_element_t *element;
     struct vfs_class *class;
     const char *local, *op;
 
-    (void) flags;
     vpath = vfs_path_new ();
 
     while ((class = _vfs_split_with_semi_skip_count (path, &local, &op, 0)) != NULL)
@@ -432,21 +432,19 @@ vfs_path_from_str_deprecated_parser (char *path, vfs_path_flag_t flags)
 */
 
 static vfs_path_t *
-vfs_path_from_str_uri_parser (char *path, vfs_path_flag_t flags)
+vfs_path_from_str_uri_parser (char *path)
 {
     vfs_path_t *vpath;
     vfs_path_element_t *element;
-
     char *url_delimiter;
 
     vpath = vfs_path_new ();
-    vpath->relative = (flags & VPF_NO_CANON) != 0;
+    vpath->relative = path != NULL && !IS_PATH_SEP (*path);
 
     while ((url_delimiter = g_strrstr (path, VFS_PATH_URL_DELIMITER)) != NULL)
     {
         char *vfs_prefix_start;
         char *real_vfs_prefix_start = url_delimiter;
-        struct vfs_s_subclass *sub = NULL;
 
         while (real_vfs_prefix_start > path && !IS_PATH_SEP (*real_vfs_prefix_start))
             real_vfs_prefix_start--;
@@ -462,8 +460,8 @@ vfs_path_from_str_uri_parser (char *path, vfs_path_flag_t flags)
         element->vfs_prefix = g_strdup (vfs_prefix_start);
 
         url_delimiter += strlen (VFS_PATH_URL_DELIMITER);
-        sub = VFSDATA (element);
-        if (sub != NULL && (sub->flags & VFS_S_REMOTE) != 0)
+
+        if (element->class != NULL && (element->class->flags & VFS_REMOTE) != 0)
         {
             char *slash_pointer;
 
@@ -532,7 +530,7 @@ static void
 vfs_path_tokens_add_class_info (const vfs_path_element_t * element, GString * ret_tokens,
                                 GString * element_tokens)
 {
-    if (((element->class->flags & VFSF_LOCAL) == 0 || ret_tokens->len > 0)
+    if (((element->class->flags & VFS_LOCAL) == 0 || ret_tokens->len > 0)
         && element_tokens->len > 0)
     {
         char *url_str;
@@ -597,7 +595,7 @@ vfs_path_strip_home (const char *dir)
 
 #define vfs_append_from_path(appendfrom, is_relative) \
 { \
-    if ((flags & VPF_STRIP_HOME) && element_index == 0 && (element->class->flags & VFSF_LOCAL) != 0) \
+    if ((flags & VPF_STRIP_HOME) && element_index == 0 && (element->class->flags & VFS_LOCAL) != 0) \
     { \
         char *stripped_home_str; \
         stripped_home_str = vfs_path_strip_home (appendfrom); \
@@ -739,9 +737,9 @@ vfs_path_from_str_flags (const char *path_str, vfs_path_flag_t flags)
         return NULL;
 
     if ((flags & VPF_USE_DEPRECATED_PARSER) != 0 && vfs_path_is_str_path_deprecated (path))
-        vpath = vfs_path_from_str_deprecated_parser (path, flags);
+        vpath = vfs_path_from_str_deprecated_parser (path);
     else
-        vpath = vfs_path_from_str_uri_parser (path, flags);
+        vpath = vfs_path_from_str_uri_parser (path);
 
     vpath->str = vfs_path_to_str_flags (vpath, 0, flags);
     g_free (path);
@@ -805,11 +803,11 @@ vfs_path_elements_count (const vfs_path_t * vpath)
  */
 
 void
-vfs_path_add_element (const vfs_path_t * vpath, const vfs_path_element_t * path_element)
+vfs_path_add_element (vfs_path_t * vpath, const vfs_path_element_t * path_element)
 {
     g_array_append_val (vpath->path, path_element);
     g_free (vpath->str);
-    ((vfs_path_t *) vpath)->str = vfs_path_to_str_flags (vpath, 0, VPF_NONE);
+    vpath->str = vfs_path_to_str_flags (vpath, 0, VPF_NONE);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1005,7 +1003,7 @@ vfs_prefix_to_class (const char *prefix)
     {
         struct vfs_class *vfs;
 
-        vfs = (struct vfs_class *) g_ptr_array_index (vfs__classes_list, i);
+        vfs = VFS_CLASS (g_ptr_array_index (vfs__classes_list, i));
         if (vfs->which != NULL)
         {
             if (vfs->which (vfs, prefix) == -1)
@@ -1094,19 +1092,18 @@ vfs_path_serialize (const vfs_path_t * vpath, GError ** mcerror)
 
     if ((vpath == NULL) || (vfs_path_elements_count (vpath) == 0))
     {
-        mc_propagate_error (mcerror, -1, "%s", "vpath object is empty");
+        mc_propagate_error (mcerror, 0, "%s", "vpath object is empty");
         return NULL;
-
     }
 
     cpath = mc_config_init (NULL, FALSE);
 
     for (element_index = 0; element_index < vfs_path_elements_count (vpath); element_index++)
     {
-        char *groupname;
+        char groupname[BUF_TINY];
         const vfs_path_element_t *element;
 
-        groupname = g_strdup_printf ("path-element-%zd", element_index);
+        g_snprintf (groupname, sizeof (groupname), "path-element-%zd", element_index);
         element = vfs_path_get_by_index (vpath, element_index);
         /* convert one element to config group */
 
@@ -1122,8 +1119,6 @@ vfs_path_serialize (const vfs_path_t * vpath, GError ** mcerror)
         mc_config_set_string_raw (cpath, groupname, "host", element->host);
         if (element->port != 0)
             mc_config_set_int (cpath, groupname, "port", element->port);
-
-        g_free (groupname);
     }
 
     ret_value = mc_serialize_config (cpath, mcerror);
@@ -1145,7 +1140,7 @@ vfs_path_t *
 vfs_path_deserialize (const char *data, GError ** mcerror)
 {
     mc_config_t *cpath;
-    size_t element_index = 0;
+    size_t element_index;
     vfs_path_t *vpath;
 
     mc_return_val_if_error (mcerror, FALSE);
@@ -1156,34 +1151,31 @@ vfs_path_deserialize (const char *data, GError ** mcerror)
 
     vpath = vfs_path_new ();
 
-    while (TRUE)
+    for (element_index = 0;; element_index++)
     {
+        struct vfs_class *eclass;
         vfs_path_element_t *element;
         char *cfg_value;
-        char *groupname;
+        char groupname[BUF_TINY];
 
-        groupname = g_strdup_printf ("path-element-%zd", element_index);
+        g_snprintf (groupname, sizeof (groupname), "path-element-%zu", element_index);
         if (!mc_config_has_group (cpath, groupname))
-        {
-            g_free (groupname);
             break;
-        }
-
-        element = g_new0 (vfs_path_element_t, 1);
 
         cfg_value = mc_config_get_string_raw (cpath, groupname, "class-name", NULL);
-        element->class = vfs_get_class_by_name (cfg_value);
-        if (element->class == NULL)
+        eclass = vfs_get_class_by_name (cfg_value);
+        if (eclass == NULL)
         {
-            g_free (element);
             vfs_path_free (vpath);
-            g_set_error (mcerror, MC_ERROR, -1, "Unable to find VFS class by name '%s'", cfg_value);
+            g_set_error (mcerror, MC_ERROR, 0, "Unable to find VFS class by name '%s'", cfg_value);
             g_free (cfg_value);
             mc_config_deinit (cpath);
             return NULL;
         }
         g_free (cfg_value);
 
+        element = g_new0 (vfs_path_element_t, 1);
+        element->class = eclass;
         element->path = mc_config_get_string_raw (cpath, groupname, "path", NULL);
 
 #ifdef HAVE_CHARSET
@@ -1200,16 +1192,13 @@ vfs_path_deserialize (const char *data, GError ** mcerror)
         element->port = mc_config_get_int (cpath, groupname, "port", 0);
 
         vpath->path = g_array_append_val (vpath->path, element);
-
-        g_free (groupname);
-        element_index++;
     }
 
     mc_config_deinit (cpath);
     if (vfs_path_elements_count (vpath) == 0)
     {
         vfs_path_free (vpath);
-        g_set_error (mcerror, MC_ERROR, -1, "No any path elements found");
+        g_set_error (mcerror, MC_ERROR, 0, "No any path elements found");
         return NULL;
     }
     vpath->str = vfs_path_to_str_flags (vpath, 0, VPF_NONE);
@@ -1272,7 +1261,7 @@ vfs_path_append_new (const vfs_path_t * vpath, const char *first_element, ...)
     va_end (args);
 
     result_str = vfs_path_as_str (vpath);
-    ret_vpath = vfs_path_build_filename (result_str, str_path, NULL);
+    ret_vpath = vfs_path_build_filename (result_str, str_path, (char *) NULL);
     g_free (str_path);
 
     return ret_vpath;
