@@ -10,7 +10,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
-#ifdef HAVE_UTIME_H
+#ifdef HAVE_UTIMENSAT
+#include <sys/time.h>
+#elif defined (HAVE_UTIME_H)
 #include <utime.h>
 #endif
 #include <stdio.h>
@@ -24,6 +26,8 @@
 
 /*** typedefs(not structures) and defined constants **********************************************/
 
+#define VFS_CLASS(a) ((struct vfs_class *) (a))
+
 #if defined (ENABLE_VFS_FTP) || defined (ENABLE_VFS_FISH) || defined (ENABLE_VFS_SMB)
 #define ENABLE_VFS_NET 1
 #endif
@@ -97,15 +101,24 @@ typedef void (*fill_names_f) (const char *);
 
 typedef void *vfsid;
 
+#ifdef HAVE_UTIMENSAT
+typedef struct timespec mc_timesbuf_t[2];
+#else
+typedef struct utimbuf mc_timesbuf_t;
+#endif
+
 /*** enums ***************************************************************************************/
 
-/* Flags of VFS classes */
 typedef enum
 {
-    VFSF_UNKNOWN = 0,
-    VFSF_LOCAL = 1 << 0,        /* Class is local (not virtual) filesystem */
-    VFSF_NOLINKS = 1 << 1       /* Hard links not supported */
-} vfs_class_flags_t;
+    VFS_UNKNOWN = 0,
+    VFS_LOCAL = 1 << 0,         /* Class is local (not virtual) filesystem */
+    VFS_NOLINKS = 1 << 1,       /* Hard links not supported */
+
+    VFS_REMOTE = 1 << 2,
+    VFS_READONLY = 1 << 3,
+    VFS_USETMP = 1 << 4
+} vfs_flags_t;
 
 /* Operations for mc_ctl - on open file */
 enum
@@ -131,10 +144,11 @@ enum
 typedef struct vfs_class
 {
     const char *name;           /* "FIles over SHell" */
-    vfs_class_flags_t flags;
+    vfs_flags_t flags;
     const char *prefix;         /* "fish:" */
-    void *data;                 /* this is for filesystem's own use */
     int verrno;                 /* can't use errno because glibc2 might define errno as function */
+    gboolean flush;             /* if set to TRUE, invalidate directory cache */
+    FILE *logfile;
 
     /* *INDENT-OFF* */
     int (*init) (struct vfs_class * me);
@@ -167,7 +181,7 @@ typedef struct vfs_class
 
     int (*chmod) (const vfs_path_t * vpath, mode_t mode);
     int (*chown) (const vfs_path_t * vpath, uid_t owner, gid_t group);
-    int (*utime) (const vfs_path_t * vpath, struct utimbuf * times);
+    int (*utime) (const vfs_path_t * vpath, mc_timesbuf_t * times);
 
     int (*readlink) (const vfs_path_t * vpath, char *buf, size_t size);
     int (*symlink) (const vfs_path_t * vpath1, const vfs_path_t * vpath2);
@@ -181,7 +195,7 @@ typedef struct vfs_class
 
     vfsid (*getid) (const vfs_path_t * vpath);
 
-    int (*nothingisopen) (vfsid id);
+    gboolean (*nothingisopen) (vfsid id);
     void (*free) (vfsid id);
 
     vfs_path_t *(*getlocalcopy) (const vfs_path_t * vpath);
@@ -217,7 +231,15 @@ extern int use_netrc;
 /*** declarations of public functions ************************************************************/
 
 /* lib/vfs/direntry.c: */
+void vfs_init_class (struct vfs_class *vclass, const char *name, vfs_flags_t flags,
+                     const char *prefix);
+
 void *vfs_s_open (const vfs_path_t * vpath, int flags, mode_t mode);
+int vfs_s_stat (const vfs_path_t * vpath, struct stat *buf);
+int vfs_s_lstat (const vfs_path_t * vpath, struct stat *buf);
+int vfs_s_fstat (void *fh, struct stat *buf);
+
+void vfs_adjust_stat (struct stat *s);
 
 vfsid vfs_getid (const vfs_path_t * vpath);
 
@@ -225,6 +247,7 @@ void vfs_init (void);
 void vfs_shut (void);
 /* Register a file system class */
 gboolean vfs_register_class (struct vfs_class *vfs);
+void vfs_unregister_class (struct vfs_class *vfs);
 
 void vfs_setup_work_dir (void);
 
@@ -242,7 +265,7 @@ gboolean vfs_file_is_local (const vfs_path_t * vpath);
 
 char *vfs_strip_suffix_from_filename (const char *filename);
 
-vfs_class_flags_t vfs_file_class_flags (const vfs_path_t * vpath);
+vfs_flags_t vfs_file_class_flags (const vfs_path_t * vpath);
 
 /* translate path back to terminal encoding, remove all #enc:
  * every invalid character is replaced with question mark
@@ -251,7 +274,7 @@ const char *vfs_translate_path (const char *path);
 /* return new string */
 char *vfs_translate_path_n (const char *path);
 
-void vfs_stamp_path (const char *path);
+void vfs_stamp_path (const vfs_path_t * path);
 
 void vfs_release_path (const vfs_path_t * vpath);
 
@@ -274,12 +297,14 @@ char *_vfs_get_cwd (void);
 
 int vfs_preallocate (int dest_desc, off_t src_fsize, off_t dest_fsize);
 
+int vfs_clone_file (int dest_vfs_fd, int src_vfs_fd);
+
 /**
  * Interface functions described in interface.c
  */
 ssize_t mc_read (int handle, void *buffer, size_t count);
 ssize_t mc_write (int handle, const void *buffer, size_t count);
-int mc_utime (const vfs_path_t * vpath, struct utimbuf *times);
+int mc_utime (const vfs_path_t * vpath, mc_timesbuf_t * times);
 int mc_readlink (const vfs_path_t * vpath, char *buf, size_t bufsiz);
 int mc_close (int handle);
 off_t mc_lseek (int fd, off_t offset, int whence);
