@@ -1,7 +1,7 @@
 /*
    Widgets for the Midnight Commander
 
-   Copyright (C) 1994-2016
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Authors:
@@ -52,7 +52,7 @@ const global_keymap_t *listbox_map = NULL;
 /*** file scope macro definitions ****************************************************************/
 
 /* Gives the position of the last item. */
-#define LISTBOX_LAST(l) (g_queue_is_empty ((l)->list) ? 0 : (int) g_queue_get_length ((l)->list) - 1)
+#define LISTBOX_LAST(l) (listbox_is_empty (l) ? 0 : (int) g_queue_get_length ((l)->list) - 1)
 
 /*** file scope type declarations ****************************************************************/
 
@@ -132,22 +132,24 @@ listbox_draw (WListbox * l, gboolean focused)
 {
     Widget *w = WIDGET (l);
     const WDialog *h = w->owner;
-    const gboolean disabled = (w->options & W_DISABLED) != 0;
-    const int normalc = disabled ? DISABLED_COLOR : h->color[DLG_COLOR_NORMAL];
-    /* *INDENT-OFF* */
-    int selc = disabled
-        ? DISABLED_COLOR
-        : focused
-            ? h->color[DLG_COLOR_HOT_FOCUS] 
-            : h->color[DLG_COLOR_FOCUS];
-    /* *INDENT-ON* */
-
+    gboolean disabled;
+    int normalc, selc;
     int length = 0;
     GList *le = NULL;
     int pos;
     int i;
     int sel_line = -1;
 
+    disabled = widget_get_state (w, WST_DISABLED);
+    normalc = disabled ? DISABLED_COLOR : h->color[DLG_COLOR_NORMAL];
+    /* *INDENT-OFF* */
+    selc = disabled
+        ? DISABLED_COLOR
+        : focused
+            ? h->color[DLG_COLOR_HOT_FOCUS] 
+            : h->color[DLG_COLOR_FOCUS];
+    /* *INDENT-ON* */
+
     if (l->list != NULL)
     {
         length = g_queue_get_length (l->list);
@@ -229,10 +231,13 @@ listbox_y_pos (WListbox * l, int y)
 static void
 listbox_fwd (WListbox * l, gboolean wrap)
 {
-    if ((guint) l->pos + 1 < g_queue_get_length (l->list))
-        listbox_select_entry (l, l->pos + 1);
-    else if (wrap)
-        listbox_select_first (l);
+    if (!listbox_is_empty (l))
+    {
+        if ((guint) l->pos + 1 < g_queue_get_length (l->list))
+            listbox_select_entry (l, l->pos + 1);
+        else if (wrap)
+            listbox_select_first (l);
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -248,10 +253,13 @@ listbox_fwd_n (WListbox * l, int n)
 static void
 listbox_back (WListbox * l, gboolean wrap)
 {
-    if (l->pos > 0)
-        listbox_select_entry (l, l->pos - 1);
-    else if (wrap)
-        listbox_select_last (l);
+    if (!listbox_is_empty (l))
+    {
+        if (l->pos > 0)
+            listbox_select_entry (l, l->pos - 1);
+        else if (wrap)
+            listbox_select_last (l);
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -400,6 +408,9 @@ listbox_do_action (WListbox * l)
 {
     int action;
 
+    if (listbox_is_empty (l))
+        return;
+
     if (l->callback != NULL)
         action = l->callback (l);
     else
@@ -442,9 +453,6 @@ listbox_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
 
     switch (msg)
     {
-    case MSG_INIT:
-        return MSG_HANDLED;
-
     case MSG_HOTKEY:
         {
             int pos;
@@ -471,12 +479,8 @@ listbox_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
         widget_move (l, l->cursor_y, 0);
         return MSG_HANDLED;
 
-    case MSG_FOCUS:
-    case MSG_UNFOCUS:
-        l->focused = msg == MSG_FOCUS;
-        /* fall through */
     case MSG_DRAW:
-        listbox_draw (l, l->focused);
+        listbox_draw (l, widget_get_state (w, WST_FOCUSED));
         return MSG_HANDLED;
 
     case MSG_DESTROY:
@@ -504,7 +508,7 @@ listbox_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
     switch (msg)
     {
     case MSG_MOUSE_DOWN:
-        dlg_select_widget (l);
+        widget_select (w);
         listbox_select_entry (l, listbox_y_pos (l, event->y));
         break;
 
@@ -552,6 +556,7 @@ listbox_new (int y, int x, int height, int width, gboolean deletable, lcback_fn
     l = g_new (WListbox, 1);
     w = WIDGET (l);
     widget_init (w, y, x, height, width, listbox_callback, listbox_mouse_callback);
+    w->options |= WOP_SELECTABLE | WOP_WANT_HOTKEY;
 
     l->list = NULL;
     l->top = l->pos = 0;
@@ -559,15 +564,15 @@ listbox_new (int y, int x, int height, int width, gboolean deletable, lcback_fn
     l->callback = callback;
     l->allow_duplicates = TRUE;
     l->scrollbar = !mc_global.tty.slow_terminal;
-    l->focused = FALSE;
-    widget_want_hotkey (w, TRUE);
-    widget_want_cursor (w, FALSE);
 
     return l;
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
+/**
+ * Finds item by its label.
+ */
 int
 listbox_search_text (WListbox * l, const char *text)
 {
@@ -590,6 +595,31 @@ listbox_search_text (WListbox * l, const char *text)
 
 /* --------------------------------------------------------------------------------------------- */
 
+/**
+ * Finds item by its 'data' slot.
+ */
+int
+listbox_search_data (WListbox * l, const void *data)
+{
+    if (!listbox_is_empty (l))
+    {
+        int i;
+        GList *le;
+
+        for (i = 0, le = g_queue_peek_head_link (l->list); le != NULL; i++, le = g_list_next (le))
+        {
+            WLEntry *e = LENTRY (le->data);
+
+            if (e->data == data)
+                return i;
+        }
+    }
+
+    return (-1);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 /* Selects the first entry and scrolls the list to the top */
 void
 listbox_select_first (WListbox * l)
@@ -757,11 +787,10 @@ listbox_remove_list (WListbox * l)
     {
         if (l->list != NULL)
         {
-            g_queue_foreach (l->list, (GFunc) listbox_entry_free, NULL);
-            g_queue_free (l->list);
+            g_queue_free_full (l->list, (GDestroyNotify) listbox_entry_free);
+            l->list = NULL;
         }
 
-        l->list = NULL;
         l->pos = l->top = 0;
     }
 }
