@@ -1,11 +1,11 @@
 /*
    Pulldown menu code
 
-   Copyright (C) 1994-2015
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Written by:
-   Andrew Borodin <aborodin@vmail.ru>, 2012, 2013
+   Andrew Borodin <aborodin@vmail.ru>, 2012, 2013, 2016
 
    This file is part of the Midnight Commander.
 
@@ -38,14 +38,16 @@
 
 #include "lib/tty/tty.h"
 #include "lib/skin.h"
-#include "lib/tty/mouse.h"
 #include "lib/tty/key.h"        /* key macros */
+#include "lib/keybind.h"        /* global_keymap_t */
 #include "lib/strutil.h"
 #include "lib/widget.h"
 #include "lib/event.h"          /* mc_event_raise() */
 
 /*** global variables ****************************************************************************/
 
+const global_keymap_t *menu_map;
+
 /*** file scope macro definitions ****************************************************************/
 
 #define MENUENTRY(x) ((menu_entry_t *)(x))
@@ -57,7 +59,7 @@ struct menu_entry_t
 {
     unsigned char first_letter;
     hotkey_t text;
-    unsigned long command;
+    long command;
     char *shortcut;
 };
 
@@ -97,7 +99,7 @@ menu_arrange (menu_t * menu, dlg_shortcut_str get_shortcut)
                 size_t len;
 
                 len = (size_t) hotkey_width (entry->text);
-                menu->max_hotkey_len = max (menu->max_hotkey_len, len);
+                menu->max_hotkey_len = MAX (menu->max_hotkey_len, len);
 
                 if (get_shortcut != NULL)
                     entry->shortcut = get_shortcut (entry->command);
@@ -105,7 +107,7 @@ menu_arrange (menu_t * menu, dlg_shortcut_str get_shortcut)
                 if (entry->shortcut != NULL)
                 {
                     len = (size_t) str_term_width1 (entry->shortcut);
-                    max_shortcut_len = max (max_shortcut_len, len);
+                    max_shortcut_len = MAX (max_shortcut_len, len);
                 }
             }
         }
@@ -199,7 +201,7 @@ menubar_draw_drop (WMenuBar * menubar)
 static void
 menubar_set_color (WMenuBar * menubar, gboolean current, gboolean hotkey)
 {
-    if (!menubar->is_active)
+    if (!widget_get_state (WIDGET (menubar), WST_FOCUSED))
         tty_setcolor (MENU_INACTIVE_COLOR);
     else if (current)
         tty_setcolor (hotkey ? MENU_HOTSEL_COLOR : MENU_SELECTED_COLOR);
@@ -216,7 +218,7 @@ menubar_draw (WMenuBar * menubar)
     GList *i;
 
     /* First draw the complete menubar */
-    tty_setcolor (menubar->is_active ? MENU_ENTRY_COLOR : MENU_INACTIVE_COLOR);
+    tty_setcolor (widget_get_state (w, WST_FOCUSED) ? MENU_ENTRY_COLOR : MENU_INACTIVE_COLOR);
     tty_draw_hline (w->y, w->x, ' ', w->cols);
 
     /* Now each one of the entries */
@@ -305,10 +307,15 @@ menubar_finish (WMenuBar * menubar)
 {
     Widget *w = WIDGET (menubar);
 
+    widget_set_state (w, WST_FOCUSED, FALSE);
     menubar->is_dropped = FALSE;
-    menubar->is_active = FALSE;
     w->lines = 1;
-    widget_want_hotkey (w, 0);
+    widget_want_hotkey (w, FALSE);
+    widget_set_options (w, WOP_SELECTABLE, FALSE);
+
+    /* Move the menubar to the bottom so that widgets displayed on top of
+     * an "invisible" menubar get the first chance to respond to mouse events. */
+    widget_set_bottom (w);
 
     dlg_select_by_id (w->owner, menubar->previous_widget);
     do_refresh ();
@@ -393,28 +400,36 @@ menubar_up (WMenuBar * menubar)
 static void
 menubar_first (WMenuBar * menubar)
 {
-    menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
-
-    if (menu->selected == 0)
-        return;
-
-    menubar_paint_idx (menubar, menu->selected, MENU_ENTRY_COLOR);
-
-    menu->selected = 0;
-
-    while (TRUE)
+    if (menubar->is_dropped)
     {
-        menu_entry_t *entry;
+        menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
 
-        entry = MENUENTRY (g_list_nth_data (menu->entries, menu->selected));
+        if (menu->selected == 0)
+            return;
 
-        if ((entry == NULL) || (entry->command == CK_IgnoreKey))
-            menu->selected++;
-        else
-            break;
+        menubar_paint_idx (menubar, menu->selected, MENU_ENTRY_COLOR);
+
+        menu->selected = 0;
+
+        while (TRUE)
+        {
+            menu_entry_t *entry;
+
+            entry = MENUENTRY (g_list_nth_data (menu->entries, menu->selected));
+
+            if ((entry == NULL) || (entry->command == CK_IgnoreKey))
+                menu->selected++;
+            else
+                break;
+        }
+
+        menubar_paint_idx (menubar, menu->selected, MENU_SELECTED_COLOR);
+    }
+    else
+    {
+        menubar->selected = 0;
+        menubar_draw (menubar);
     }
-
-    menubar_paint_idx (menubar, menu->selected, MENU_SELECTED_COLOR);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -422,46 +437,92 @@ menubar_first (WMenuBar * menubar)
 static void
 menubar_last (WMenuBar * menubar)
 {
-    menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
-    const unsigned int len = g_list_length (menu->entries);
-    menu_entry_t *entry;
-
-    if (menu->selected == len - 1)
-        return;
-
-    menubar_paint_idx (menubar, menu->selected, MENU_ENTRY_COLOR);
-
-    menu->selected = len;
-
-    do
+    if (menubar->is_dropped)
     {
-        menu->selected--;
-        entry = MENUENTRY (g_list_nth_data (menu->entries, menu->selected));
-    }
-    while ((entry == NULL) || (entry->command == CK_IgnoreKey));
+        menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
+        const unsigned int len = g_list_length (menu->entries);
+        menu_entry_t *entry;
 
-    menubar_paint_idx (menubar, menu->selected, MENU_SELECTED_COLOR);
+        if (menu->selected == len - 1)
+            return;
+
+        menubar_paint_idx (menubar, menu->selected, MENU_ENTRY_COLOR);
+
+        menu->selected = len;
+
+        do
+        {
+            menu->selected--;
+            entry = MENUENTRY (g_list_nth_data (menu->entries, menu->selected));
+        }
+        while ((entry == NULL) || (entry->command == CK_IgnoreKey));
+
+        menubar_paint_idx (menubar, menu->selected, MENU_SELECTED_COLOR);
+    }
+    else
+    {
+        menubar->selected = g_list_length (menubar->menu) - 1;
+        menubar_draw (menubar);
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
-menubar_handle_key (WMenuBar * menubar, int key)
+static cb_ret_t
+menubar_try_drop_menu (WMenuBar * menubar, int hotkey)
 {
-    /* Lowercase */
-    if (isascii (key))
-        key = g_ascii_tolower (key);
+    GList *i;
 
-    if (is_abort_char (key))
+    for (i = menubar->menu; i != NULL; i = g_list_next (i))
     {
-        menubar_finish (menubar);
-        return 1;
+        menu_t *menu = MENU (i->data);
+
+        if (menu->text.hotkey != NULL && hotkey == g_ascii_tolower (menu->text.hotkey[0]))
+        {
+            menubar_drop (menubar, g_list_position (menubar->menu, i));
+            return MSG_HANDLED;
+        }
     }
 
-    /* menubar help or menubar navigation */
-    switch (key)
+    return MSG_NOT_HANDLED;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static cb_ret_t
+menubar_try_exec_menu (WMenuBar * menubar, int hotkey)
+{
+    menu_t *menu;
+    GList *i;
+
+    menu = g_list_nth_data (menubar->menu, menubar->selected);
+
+    for (i = menu->entries; i != NULL; i = g_list_next (i))
     {
-    case KEY_F (1):
+        const menu_entry_t *entry = MENUENTRY (i->data);
+
+        if (entry != NULL && entry->text.hotkey != NULL
+            && hotkey == g_ascii_tolower (entry->text.hotkey[0]))
+        {
+            menu->selected = g_list_position (menu->entries, i);
+            menubar_execute (menubar);
+            return MSG_HANDLED;
+        }
+    }
+
+    return MSG_NOT_HANDLED;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static cb_ret_t
+menubar_execute_cmd (WMenuBar * menubar, unsigned long command)
+{
+    cb_ret_t ret = MSG_HANDLED;
+
+    switch (command)
+    {
+    case CK_Help:
         {
             ev_help_t event_data = { NULL, NULL };
 
@@ -473,91 +534,99 @@ menubar_handle_key (WMenuBar * menubar, int key)
 
             mc_event_raise (MCEVENT_GROUP_CORE, "help", &event_data);
             menubar_draw (menubar);
-            return 1;
         }
-    case KEY_LEFT:
-    case XCTRL ('b'):
+        break;
+
+    case CK_Left:
         menubar_left (menubar);
-        return 1;
-
-    case KEY_RIGHT:
-    case XCTRL ('f'):
+        break;
+    case CK_Right:
         menubar_right (menubar);
-        return 1;
-    }
-
-    if (!menubar->is_dropped)
-    {
-        GList *i;
-
-        /* drop menu by hotkey */
-        for (i = menubar->menu; i != NULL; i = g_list_next (i))
-        {
-            menu_t *menu = MENU (i->data);
-
-            if ((menu->text.hotkey != NULL) && (key == g_ascii_tolower (menu->text.hotkey[0])))
-            {
-                menubar_drop (menubar, g_list_position (menubar->menu, i));
-                return 1;
-            }
-        }
-
-        /* drop menu by Enter or Dowwn key */
-        if (key == KEY_ENTER || key == XCTRL ('n') || key == KEY_DOWN || key == '\n')
-            menubar_drop (menubar, menubar->selected);
-
-        return 1;
-    }
-
-    {
-        menu_t *menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
-        GList *i;
-
-        /* execute menu command by hotkey */
-        for (i = menu->entries; i != NULL; i = g_list_next (i))
-        {
-            const menu_entry_t *entry = MENUENTRY (i->data);
-
-            if ((entry != NULL) && (entry->command != CK_IgnoreKey)
-                && (entry->text.hotkey != NULL) && (key == g_ascii_tolower (entry->text.hotkey[0])))
-            {
-                menu->selected = g_list_position (menu->entries, i);
-                menubar_execute (menubar);
-                return 1;
-            }
-        }
-
-        /* menu execute by Enter or menu navigation */
-        switch (key)
-        {
-        case KEY_ENTER:
-        case '\n':
-            menubar_execute (menubar);
-            return 1;
-
-        case KEY_HOME:
-        case ALT ('<'):
-            menubar_first (menubar);
-            break;
-
-        case KEY_END:
-        case ALT ('>'):
-            menubar_last (menubar);
-            break;
-
-        case KEY_DOWN:
-        case XCTRL ('n'):
-            menubar_down (menubar);
-            break;
-
-        case KEY_UP:
-        case XCTRL ('p'):
+        break;
+    case CK_Up:
+        if (menubar->is_dropped)
             menubar_up (menubar);
-            break;
-        }
+        break;
+    case CK_Down:
+        if (menubar->is_dropped)
+            menubar_down (menubar);
+        else
+            menubar_drop (menubar, menubar->selected);
+        break;
+    case CK_Home:
+        menubar_first (menubar);
+        break;
+    case CK_End:
+        menubar_last (menubar);
+        break;
+
+    case CK_Enter:
+        if (menubar->is_dropped)
+            menubar_execute (menubar);
+        else
+            menubar_drop (menubar, menubar->selected);
+        break;
+    case CK_Quit:
+        menubar_finish (menubar);
+        break;
+
+    default:
+        ret = MSG_NOT_HANDLED;
+        break;
     }
 
-    return 0;
+    return ret;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static int
+menubar_handle_key (WMenuBar * menubar, int key)
+{
+    unsigned long cmd;
+    cb_ret_t ret = MSG_NOT_HANDLED;
+
+    cmd = keybind_lookup_keymap_command (menu_map, key);
+
+    if (cmd != CK_IgnoreKey)
+        ret = menubar_execute_cmd (menubar, cmd);
+
+    if (ret != MSG_HANDLED)
+    {
+        if (menubar->is_dropped)
+            ret = menubar_try_exec_menu (menubar, key);
+        else
+            ret = menubar_try_drop_menu (menubar, key);
+    }
+
+    return ret;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static gboolean
+menubar_refresh (WMenuBar * menubar)
+{
+    Widget *w = WIDGET (menubar);
+
+    if (!widget_get_state (w, WST_FOCUSED))
+        return FALSE;
+
+    /* Trick to get all the mouse events */
+    w->lines = LINES;
+
+    /* Trick to get all of the hotkeys */
+    widget_want_hotkey (w, TRUE);
+    return TRUE;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+menubar_free_menu (WMenuBar * menubar)
+{
+    if (menubar->menu != NULL)
+        g_list_free_full (menubar->menu, (GDestroyNotify) destroy_menu);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -571,21 +640,20 @@ menubar_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
     {
         /* We do not want the focus unless we have been activated */
     case MSG_FOCUS:
-        if (!menubar->is_active)
-            return MSG_NOT_HANDLED;
+        if (menubar_refresh (menubar))
+        {
+            menubar_draw (menubar);
+            return MSG_HANDLED;
+        }
+        return MSG_NOT_HANDLED;
 
-        /* Trick to get all the mouse events */
-        w->lines = LINES;
-
-        /* Trick to get all of the hotkeys */
-        widget_want_hotkey (w, 1);
-        menubar_draw (menubar);
-        return MSG_HANDLED;
+    case MSG_UNFOCUS:
+        return widget_get_state (w, WST_FOCUSED) ? MSG_NOT_HANDLED : MSG_HANDLED;
 
         /* We don't want the buttonbar to activate while using the menubar */
     case MSG_HOTKEY:
     case MSG_KEY:
-        if (menubar->is_active)
+        if (widget_get_state (w, WST_FOCUSED))
         {
             menubar_handle_key (menubar, parm);
             return MSG_HANDLED;
@@ -596,25 +664,18 @@ menubar_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
         /* Put the cursor in a suitable place */
         return MSG_NOT_HANDLED;
 
-    case MSG_UNFOCUS:
-        return menubar->is_active ? MSG_NOT_HANDLED : MSG_HANDLED;
-
     case MSG_DRAW:
-        if (menubar->is_visible)
-        {
+        if (menubar->is_visible || menubar_refresh (menubar))
             menubar_draw (menubar);
-            return MSG_HANDLED;
-        }
-        /* fall through */
+        return MSG_HANDLED;
 
     case MSG_RESIZE:
         /* try show menu after screen resize */
-        send_message (w, sender, MSG_FOCUS, 0, data);
+        menubar_refresh (menubar);
         return MSG_HANDLED;
 
-
     case MSG_DESTROY:
-        menubar_set_menu (menubar, NULL);
+        menubar_free_menu (menubar);
         return MSG_HANDLED;
 
     default:
@@ -624,142 +685,181 @@ menubar_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
-menubar_event (Gpm_Event * event, void *data)
+static unsigned int
+menubar_get_menu_by_x_coord (const WMenuBar * menubar, int x)
 {
-    WMenuBar *menubar = MENUBAR (data);
-    Widget *w = WIDGET (data);
-    gboolean was_active = TRUE;
-    int left_x, right_x, bottom_y;
+    unsigned int i;
+    GList *menu;
+
+    for (i = 0, menu = menubar->menu;
+         menu != NULL && x > MENU (menu->data)->start_x; i++, menu = g_list_next (menu))
+        ;
+
+    /* Don't set the invalid value -1 */
+    if (i != 0)
+        i--;
+
+    return i;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static gboolean
+menubar_mouse_on_menu (const WMenuBar * menubar, int y, int x)
+{
+    Widget *w = WIDGET (menubar);
     menu_t *menu;
-    Gpm_Event local;
-
-    if (!mouse_global_in_widget (event, w))
-        return MOU_UNHANDLED;
-
-    /* ignore unsupported events */
-    if ((event->type & (GPM_UP | GPM_DOWN | GPM_DRAG)) == 0)
-        return MOU_NORMAL;
-
-    /* ignore wheel events if menu is inactive */
-    if (!menubar->is_active && ((event->buttons & (GPM_B_MIDDLE | GPM_B_UP | GPM_B_DOWN)) != 0))
-        return MOU_NORMAL;
-
-    local = mouse_get_local (event, w);
-
-    if (local.y == 1 && (local.type & GPM_UP) != 0)
-        return MOU_NORMAL;
+    int left_x, right_x, bottom_y;
 
     if (!menubar->is_dropped)
-    {
-        if (local.y > 1)
-        {
-            /* mouse click below menubar -- close menu and send focus to widget under mouse */
-            menubar_finish (menubar);
-            return MOU_UNHANDLED;
-        }
+        return FALSE;
 
-        menubar->previous_widget = dlg_get_current_widget_id (w->owner);
-        menubar->is_active = TRUE;
-        menubar->is_dropped = TRUE;
-        was_active = FALSE;
-    }
-
-    /* Mouse operations on the menubar */
-    if (local.y == 1 || !was_active)
-    {
-        /* wheel events on menubar */
-        if ((local.buttons & GPM_B_UP) != 0)
-            menubar_left (menubar);
-        else if ((local.buttons & GPM_B_DOWN) != 0)
-            menubar_right (menubar);
-        else
-        {
-            const unsigned int len = g_list_length (menubar->menu);
-            unsigned int new_selection = 0;
-
-            while ((new_selection < len)
-                   && (local.x > MENU (g_list_nth_data (menubar->menu, new_selection))->start_x))
-                new_selection++;
-
-            if (new_selection != 0)     /* Don't set the invalid value -1 */
-                new_selection--;
-
-            if (!was_active)
-            {
-                menubar->selected = new_selection;
-                dlg_select_widget (menubar);
-            }
-            else
-            {
-                menubar_remove (menubar);
-                menubar->selected = new_selection;
-            }
-            menubar_draw (menubar);
-        }
-        return MOU_NORMAL;
-    }
-
-    if (!menubar->is_dropped || (local.y < 2))
-        return MOU_NORMAL;
-
-    /* middle click -- everywhere */
-    if (((local.buttons & GPM_B_MIDDLE) != 0) && ((local.type & GPM_DOWN) != 0))
-    {
-        menubar_execute (menubar);
-        return MOU_NORMAL;
-    }
-
-    /* the mouse operation is on the menus or it is not */
     menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
     left_x = menu->start_x;
     right_x = left_x + menu->max_entry_len + 3;
     if (right_x > w->cols)
     {
-        left_x = w->cols - menu->max_entry_len - 3;
+        left_x = w->cols - (menu->max_entry_len + 3);
         right_x = w->cols;
     }
 
-    bottom_y = g_list_length (menu->entries) + 3;
+    bottom_y = g_list_length (menu->entries) + 2;       /* skip bar and top frame */
 
-    if ((local.x >= left_x) && (local.x <= right_x) && (local.y <= bottom_y))
+    return (x >= left_x && x < right_x && y > 1 && y < bottom_y);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+menubar_change_selected_item (WMenuBar * menubar, int y)
+{
+    menu_t *menu;
+    menu_entry_t *entry;
+
+    y -= 2;                     /* skip bar and top frame */
+    menu = MENU (g_list_nth_data (menubar->menu, menubar->selected));
+    entry = MENUENTRY (g_list_nth_data (menu->entries, y));
+
+    if (entry != NULL && entry->command != CK_IgnoreKey)
     {
-        int pos = local.y - 3;
-        const menu_entry_t *entry = MENUENTRY (g_list_nth_data (menu->entries, pos));
-
-        /* mouse wheel */
-        if ((local.buttons & GPM_B_UP) != 0 && (local.type & GPM_DOWN) != 0)
-        {
-            menubar_up (menubar);
-            return MOU_NORMAL;
-        }
-        if ((local.buttons & GPM_B_DOWN) != 0 && (local.type & GPM_DOWN) != 0)
-        {
-            menubar_down (menubar);
-            return MOU_NORMAL;
-        }
-
-        /* ignore events above and below dropped down menu */
-        if ((pos < 0) || (pos >= bottom_y - 3))
-            return MOU_NORMAL;
-
-        if ((entry != NULL) && (entry->command != CK_IgnoreKey))
-        {
-            menubar_paint_idx (menubar, menu->selected, MENU_ENTRY_COLOR);
-            menu->selected = pos;
-            menubar_paint_idx (menubar, menu->selected, MENU_SELECTED_COLOR);
-
-            if ((event->type & GPM_UP) != 0)
-                menubar_execute (menubar);
-        }
+        menubar_paint_idx (menubar, menu->selected, MENU_ENTRY_COLOR);
+        menu->selected = y;
+        menubar_paint_idx (menubar, menu->selected, MENU_SELECTED_COLOR);
     }
-    else if (((local.type & GPM_DOWN) != 0) && ((local.buttons & (GPM_B_UP | GPM_B_DOWN)) == 0))
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+menubar_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
+{
+    static gboolean was_drag = FALSE;
+
+    WMenuBar *menubar = MENUBAR (w);
+    gboolean mouse_on_drop;
+
+    mouse_on_drop = menubar_mouse_on_menu (menubar, event->y, event->x);
+
+    switch (msg)
     {
-        /* use click not wheel to close menu */
-        menubar_finish (menubar);
-    }
+    case MSG_MOUSE_DOWN:
+        was_drag = FALSE;
 
-    return MOU_NORMAL;
+        if (event->y == 0)
+        {
+            /* events on menubar */
+            unsigned int selected;
+
+            selected = menubar_get_menu_by_x_coord (menubar, event->x);
+            menubar_activate (menubar, TRUE, selected);
+            menubar_remove (menubar);   /* if already shown */
+            menubar_drop (menubar, selected);
+        }
+        else if (mouse_on_drop)
+            menubar_change_selected_item (menubar, event->y);
+        else
+        {
+            /* mouse click outside menubar or dropdown -- close menu */
+            menubar_finish (menubar);
+
+            /*
+             * @FIXME.
+             *
+             * Unless we clear the 'capture' flag, we'll receive MSG_MOUSE_DRAG
+             * events belonging to this click (in case the user drags the mouse,
+             * of course).
+             *
+             * For the time being, we mark this with FIXME as this flag should
+             * preferably be regarded as "implementation detail" and not be
+             * touched by us. We should think of some other way of communicating
+             * this to the system.
+             */
+            w->mouse.capture = FALSE;
+        }
+        break;
+
+    case MSG_MOUSE_UP:
+        if (was_drag && mouse_on_drop)
+            menubar_execute (menubar);
+        was_drag = FALSE;
+        break;
+
+    case MSG_MOUSE_CLICK:
+        was_drag = FALSE;
+
+        if ((event->buttons & GPM_B_MIDDLE) != 0 && event->y > 0 && menubar->is_dropped)
+        {
+            /* middle click -- everywhere */
+            menubar_execute (menubar);
+        }
+        else if (mouse_on_drop)
+            menubar_execute (menubar);
+        else if (event->y > 0)
+            /* releasing the mouse button outside the menu -- close menu */
+            menubar_finish (menubar);
+        break;
+
+    case MSG_MOUSE_DRAG:
+        if (event->y == 0)
+        {
+            menubar_remove (menubar);
+            menubar_drop (menubar, menubar_get_menu_by_x_coord (menubar, event->x));
+        }
+        else if (mouse_on_drop)
+            menubar_change_selected_item (menubar, event->y);
+
+        was_drag = TRUE;
+        break;
+
+    case MSG_MOUSE_SCROLL_UP:
+    case MSG_MOUSE_SCROLL_DOWN:
+        was_drag = FALSE;
+
+        if (widget_get_state (w, WST_FOCUSED))
+        {
+            if (event->y == 0)
+            {
+                /* menubar: left/right */
+                if (msg == MSG_MOUSE_SCROLL_UP)
+                    menubar_left (menubar);
+                else
+                    menubar_right (menubar);
+            }
+            else if (mouse_on_drop)
+            {
+                /* drop-down menu: up/down */
+                if (msg == MSG_MOUSE_SCROLL_UP)
+                    menubar_up (menubar);
+                else
+                    menubar_down (menubar);
+            }
+        }
+        break;
+
+    default:
+        was_drag = FALSE;
+        break;
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -767,7 +867,7 @@ menubar_event (Gpm_Event * event, void *data)
 /* --------------------------------------------------------------------------------------------- */
 
 menu_entry_t *
-menu_entry_create (const char *name, unsigned long command)
+menu_entry_create (const char *name, long command)
 {
     menu_entry_t *entry;
 
@@ -842,10 +942,11 @@ menubar_new (int y, int x, int cols, GList * menu, gboolean visible)
 
     menubar = g_new0 (WMenuBar, 1);
     w = WIDGET (menubar);
-    widget_init (w, y, x, 1, cols, menubar_callback, menubar_event);
-
+    widget_init (w, y, x, 1, cols, menubar_callback, menubar_mouse_callback);
+    /* initially, menubar is not selectable */
+    widget_set_options (w, WOP_SELECTABLE, FALSE);
+    w->options |= WOP_TOP_SELECT;
     menubar->is_visible = visible;
-    widget_want_cursor (w, FALSE);
     menubar_set_menu (menubar, menu);
 
     return menubar;
@@ -857,15 +958,13 @@ void
 menubar_set_menu (WMenuBar * menubar, GList * menu)
 {
     /* delete previous menu */
-    if (menubar->menu != NULL)
-        g_list_free_full (menubar->menu, (GDestroyNotify) destroy_menu);
-
+    menubar_free_menu (menubar);
     /* add new menu */
-    menubar->is_active = FALSE;
     menubar->is_dropped = FALSE;
     menubar->menu = menu;
     menubar->selected = 0;
     menubar_arrange (menubar);
+    widget_set_state (WIDGET (menubar), WST_FOCUSED, FALSE);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -944,3 +1043,34 @@ find_menubar (const WDialog * h)
 }
 
 /* --------------------------------------------------------------------------------------------- */
+
+/**
+ * Activate menu bar.
+ *
+ * @param menubar menu bar object
+ * @param dropped whether dropdown menus should be drooped or not
+ * @which number of active dropdown menu
+ */
+void
+menubar_activate (WMenuBar * menubar, gboolean dropped, int which)
+{
+    Widget *w = WIDGET (menubar);
+
+    if (!widget_get_state (w, WST_FOCUSED))
+    {
+        widget_set_options (w, WOP_SELECTABLE, TRUE);
+
+        widget_set_state (w, WST_FOCUSED, TRUE);        /* FIXME: unneeded? */
+        menubar->is_dropped = dropped;
+        if (which >= 0)
+            menubar->selected = (guint) which;
+
+        menubar->previous_widget = dlg_get_current_widget_id (w->owner);
+
+        /* Bring it to the top so it receives all mouse events before any other widget.
+         * See also comment in menubar_finish(). */
+        widget_select (w);
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
