@@ -1,7 +1,7 @@
 /*
    Client interface for General purpose Linux console save/restore server
 
-   Copyright (C) 1994-2016
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
@@ -42,7 +42,7 @@
 
 #include "lib/unixcompat.h"
 #include "lib/tty/tty.h"
-#include "lib/skin.h"           /* tty_set_normal_attrs */
+#include "lib/tty/color.h"      /* tty_set_normal_attrs */
 #include "lib/tty/win.h"
 #include "lib/util.h"           /* mc_build_filename() */
 
@@ -408,7 +408,7 @@ show_console_contents_freebsd (int starty, unsigned char begin_line, unsigned ch
     for (line = begin_line; line <= end_line; line++)
     {
         tty_gotoyx (starty + line - begin_line, 0);
-        for (col = 0; col < min (COLS, screen_info.mv_csz); col++)
+        for (col = 0; col < MIN (COLS, screen_info.mv_csz); col++)
         {
             c = screen_shot.buf[line * screen_info.mv_csz + col] & 0xFF;
             tty_print_char (c);
