@@ -1,7 +1,7 @@
 /*
    Execution routines for GNU Midnight Commander
 
-   Copyright (C) 2003-2014
+   Copyright (C) 2003-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -49,7 +49,7 @@
 #include "filemanager/layout.h" /* use_dash() */
 #include "consaver/cons.saver.h"
 #ifdef ENABLE_SUBSHELL
-#include "subshell.h"
+#include "subshell/subshell.h"
 #endif
 #include "setup.h"              /* clear_before_exec */
 
@@ -78,7 +78,7 @@ char *execute_get_external_cmd_opts_from_config (const char *command,
 static void
 edition_post_exec (void)
 {
-    do_enter_ca_mode ();
+    tty_enter_ca_mode ();
 
     /* FIXME: Missing on slang endwin? */
     tty_reset_prog_mode ();
@@ -122,7 +122,7 @@ edition_pre_exec (void)
      * Do not move this before endwin: in some systems rmcup includes
      * a call to clear screen, so it will end up clearing the shell screen.
      */
-    do_exit_ca_mode ();
+    tty_exit_ca_mode ();
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -132,7 +132,7 @@ static void
 do_possible_cd (const vfs_path_t * new_dir_vpath)
 {
     if (!do_cd (new_dir_vpath, cd_exact))
-        message (D_ERROR, _("Warning"),
+        message (D_ERROR, _("Warning"), "%s",
                  _("The Commander can't change to the directory that\n"
                    "the subshell claims you are in. Perhaps you have\n"
                    "deleted your working directory, or given yourself\n"
@@ -233,7 +233,8 @@ execute_get_opts_from_cfg (const char *command, const char *default_str)
     char *str_from_config;
 
     str_from_config =
-        mc_config_get_string_raw (mc_main_config, CONFIG_EXT_EDITOR_VIEWER_SECTION, command, NULL);
+        mc_config_get_string_raw (mc_global.main_config, CONFIG_EXT_EDITOR_VIEWER_SECTION, command,
+                                  NULL);
 
     if (str_from_config == NULL)
     {
@@ -333,7 +334,7 @@ do_executev (const char *shell, int flags, char *const argv[])
 #endif /* ENABLE_SUBSHELL */
             )
         {
-            printf (_("Press any key to continue..."));
+            printf ("%s", _("Press any key to continue..."));
             fflush (stdout);
             tty_raw_mode ();
             get_key_code (0);
@@ -429,27 +430,21 @@ shell_execute (const char *command, int flags)
 
 #ifdef ENABLE_SUBSHELL
     if (mc_global.tty.use_subshell)
+    {
         if (subshell_state == INACTIVE)
-            do_execute (mc_global.tty.shell, cmd ? cmd : command, flags | EXECUTE_AS_SHELL);
+            do_execute (mc_global.shell->path, cmd ? cmd : command, flags | EXECUTE_AS_SHELL);
         else
-            message (D_ERROR, MSG_ERROR, _("The shell is already running a command"));
+            message (D_ERROR, MSG_ERROR, "%s", _("The shell is already running a command"));
+    }
     else
 #endif /* ENABLE_SUBSHELL */
-        do_execute (mc_global.tty.shell, cmd ? cmd : command, flags | EXECUTE_AS_SHELL);
+        do_execute (mc_global.shell->path, cmd ? cmd : command, flags | EXECUTE_AS_SHELL);
 
     g_free (cmd);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-void
-exec_shell (void)
-{
-    do_execute (mc_global.tty.shell, 0, 0);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 void
 toggle_panels (void)
 {
@@ -475,7 +470,7 @@ toggle_panels (void)
     tty_noecho ();
     tty_keypad (FALSE);
     tty_reset_screen ();
-    do_exit_ca_mode ();
+    tty_exit_ca_mode ();
     tty_raw_mode ();
     if (mc_global.tty.console_flag != '\0')
         handle_console (CONSOLE_RESTORE);
@@ -493,10 +488,10 @@ toggle_panels (void)
     {
         if (output_starts_shell)
         {
-            fprintf (stderr, _("Type 'exit' to return to the Midnight Commander"));
-            fprintf (stderr, "\n\r\n\r");
+            fputs (_("Type 'exit' to return to the Midnight Commander"), stderr);
+            fputs ("\n\r\n\r", stderr);
 
-            my_system (EXECUTE_INTERNAL, mc_global.tty.shell, NULL);
+            my_system (EXECUTE_INTERNAL, mc_global.shell->path, NULL);
         }
         else
             get_key_code (0);
@@ -505,7 +500,7 @@ toggle_panels (void)
     if (mc_global.tty.console_flag != '\0')
         handle_console (CONSOLE_SAVE);
 
-    do_enter_ca_mode ();
+    tty_enter_ca_mode ();
 
     tty_reset_prog_mode ();
     tty_keypad (TRUE);
@@ -629,7 +624,7 @@ execute_external_editor_or_viewer (const char *command, const vfs_path_t * filen
     vfs_path_t *localcopy_vpath = NULL;
     const vfs_path_t *do_execute_vpath;
     char *extern_cmd_options;
-    time_t mtime;
+    time_t mtime = 0;
 
     if (!execute_prepare_with_vfs_arg (filename_vpath, &localcopy_vpath, &mtime))
         return;
@@ -653,7 +648,6 @@ execute_external_editor_or_viewer (const char *command, const vfs_path_t * filen
             do_executev (command, EXECUTE_INTERNAL, NULL);
 
         g_free (extern_cmd_options);
-
     }
 
     execute_cleanup_with_vfs_arg (filename_vpath, &localcopy_vpath, &mtime);
