@@ -2,7 +2,7 @@
    Routines invoked by a function key
    They normally operate on the current panel.
 
-   Copyright (C) 1994-2018
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -428,9 +428,6 @@ nice_cd (const char *text, const char *xtext, const char *help,
     char *machine;
     char *cd_path;
 
-    if (!SELECTED_IS_PANEL)
-        return;
-
     machine =
         input_dialog_help (text, xtext, help, history_name, INPUT_LAST_TEXT, strip_password,
                            INPUT_COMPLETE_FILENAMES | INPUT_COMPLETE_CD | INPUT_COMPLETE_HOSTNAMES |
@@ -457,11 +454,22 @@ nice_cd (const char *text, const char *xtext, const char *help,
     }
 
     {
+        panel_view_mode_t save_type;
         vfs_path_t *cd_vpath;
 
+        save_type = get_panel_type (MENU_PANEL_IDX);
+
+        if (save_type != view_listing)
+            create_panel (MENU_PANEL_IDX, view_listing);
+
         cd_vpath = vfs_path_from_str_flags (cd_path, VPF_NO_CANON);
         if (!do_panel_cd (MENU_PANEL, cd_vpath, cd_parse_command))
+        {
             message (D_ERROR, MSG_ERROR, _("Cannot chdir to \"%s\""), cd_path);
+
+            if (save_type != view_listing)
+                create_panel (MENU_PANEL_IDX, save_type);
+        }
         vfs_path_free (cd_vpath);
     }
     g_free (cd_path);
@@ -506,8 +514,8 @@ configure_panel_listing (WPanel * p, int list_format, int brief_cols, gboolean u
 static void
 switch_to_listing (int panel_index)
 {
-    if (get_display_type (panel_index) != view_listing)
-        set_display_type (panel_index, view_listing);
+    if (get_panel_type (panel_index) != view_listing)
+        create_panel (panel_index, view_listing);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1527,12 +1535,12 @@ save_setup_cmd (void)
 void
 info_cmd_no_menu (void)
 {
-    if (get_display_type (0) == view_info)
-        set_display_type (0, view_listing);
-    else if (get_display_type (1) == view_info)
-        set_display_type (1, view_listing);
+    if (get_panel_type (0) == view_info)
+        create_panel (0, view_listing);
+    else if (get_panel_type (1) == view_info)
+        create_panel (1, view_listing);
     else
-        set_display_type (current_panel == left_panel ? 1 : 0, view_info);
+        create_panel (current_panel == left_panel ? 1 : 0, view_info);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1540,12 +1548,12 @@ info_cmd_no_menu (void)
 void
 quick_cmd_no_menu (void)
 {
-    if (get_display_type (0) == view_quick)
-        set_display_type (0, view_listing);
-    else if (get_display_type (1) == view_quick)
-        set_display_type (1, view_listing);
+    if (get_panel_type (0) == view_quick)
+        create_panel (0, view_listing);
+    else if (get_panel_type (1) == view_quick)
+        create_panel (1, view_listing);
     else
-        set_display_type (current_panel == left_panel ? 1 : 0, view_quick);
+        create_panel (current_panel == left_panel ? 1 : 0, view_quick);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1593,7 +1601,7 @@ setup_listing_format_cmd (void)
 void
 panel_tree_cmd (void)
 {
-    set_display_type (MENU_PANEL_IDX, view_tree);
+    create_panel (MENU_PANEL_IDX, view_tree);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1601,7 +1609,7 @@ panel_tree_cmd (void)
 void
 info_cmd (void)
 {
-    set_display_type (MENU_PANEL_IDX, view_info);
+    create_panel (MENU_PANEL_IDX, view_info);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1611,7 +1619,7 @@ quick_view_cmd (void)
 {
     if (PANEL (get_panel_widget (MENU_PANEL_IDX)) == current_panel)
         change_panel ();
-    set_display_type (MENU_PANEL_IDX, view_quick);
+    create_panel (MENU_PANEL_IDX, view_quick);
 }
 
 /* --------------------------------------------------------------------------------------------- */
