@@ -1,7 +1,7 @@
 /*
    Directory routines
 
-   Copyright (C) 1994-2015
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -47,6 +47,7 @@
 #include "src/setup.h"          /* panels_options */
 
 #include "treestore.h"
+#include "file.h"               /* file_is_symlink_to_dir() */
 #include "dir.h"
 #include "layout.h"             /* rotate_dash() */
 
@@ -55,9 +56,9 @@
 /*** file scope macro definitions ****************************************************************/
 
 #define MY_ISDIR(x) (\
-    (is_exe (x->st.st_mode) && !(S_ISDIR (x->st.st_mode) || x->f.link_to_dir) && exec_first) \
+    (is_exe (x->st.st_mode) && !(S_ISDIR (x->st.st_mode) || link_isdir (x)) && exec_first) \
         ? 1 \
-        : ( (S_ISDIR (x->st.st_mode) || x->f.link_to_dir) ? 2 : 0) )
+        : ( (S_ISDIR (x->st.st_mode) || link_isdir (x)) ? 2 : 0) )
 
 /*** file scope type declarations ****************************************************************/
 
@@ -67,7 +68,7 @@
 static int reverse = 1;
 
 /* Are the files sorted case sensitively? */
-static int case_sensitive = OS_SORT_CASE_SENSITIVE_DEFAULT;
+static gboolean case_sensitive = OS_SORT_CASE_SENSITIVE_DEFAULT;
 
 /* Are the exec_bit files top in list */
 static gboolean exec_first = TRUE;
@@ -150,6 +151,7 @@ handle_dirent (struct dirent *dp, const char *fltr, struct stat *buf1, int *link
                int *stale_link)
 {
     vfs_path_t *vpath;
+    gboolean stale;
 
     if (DIR_IS_DOT (dp->d_name) || DIR_IS_DOTDOT (dp->d_name))
         return FALSE;
@@ -173,17 +175,8 @@ handle_dirent (struct dirent *dp, const char *fltr, struct stat *buf1, int *link
         tree_store_mark_checked (dp->d_name);
 
     /* A link to a file or a directory? */
-    *link_to_dir = 0;
-    *stale_link = 0;
-    if (S_ISLNK (buf1->st_mode))
-    {
-        struct stat buf2;
-
-        if (mc_stat (vpath, &buf2) == 0)
-            *link_to_dir = S_ISDIR (buf2.st_mode) != 0;
-        else
-            *stale_link = 1;
-    }
+    *link_to_dir = file_is_symlink_to_dir (vpath, buf1, &stale) ? 1 : 0;
+    *stale_link = stale ? 1 : 0;
 
     vfs_path_free (vpath);
 
@@ -208,7 +201,7 @@ dir_get_dotdot_stat (const vfs_path_t * vpath, struct stat *st)
         {
             vfs_path_t *tmp_vpath;
 
-            tmp_vpath = vfs_path_append_new (vpath, "..", NULL);
+            tmp_vpath = vfs_path_append_new (vpath, "..", (char *) NULL);
             ret = mc_stat (tmp_vpath, st) == 0;
             vfs_path_free (tmp_vpath);
         }
@@ -225,18 +218,7 @@ alloc_dir_copy (int size)
     if (dir_copy.size < size)
     {
         if (dir_copy.list != NULL)
-        {
-            int i;
-
-            for (i = 0; i < dir_copy.len; i++)
-            {
-                file_entry_t *fentry;
-
-                fentry = &(dir_copy.list)[i];
-                g_free (fentry->fname);
-            }
-            g_free (dir_copy.list);
-        }
+            dir_list_free_list (&dir_copy);
 
         dir_copy.list = g_new0 (file_entry_t, size);
         dir_copy.size = size;
@@ -260,7 +242,7 @@ gboolean
 dir_list_grow (dir_list * list, int delta)
 {
     int size;
-    gboolean clear = FALSE;
+    gboolean clear_flag = FALSE;
 
     if (list == NULL)
         return FALSE;
@@ -272,7 +254,7 @@ dir_list_grow (dir_list * list, int delta)
     if (size <= 0)
     {
         size = DIR_LIST_MIN_SIZE;
-        clear = TRUE;
+        clear_flag = TRUE;
     }
 
     if (size != list->size)
@@ -287,7 +269,7 @@ dir_list_grow (dir_list * list, int delta)
         list->size = size;
     }
 
-    list->len = clear ? 0 : min (list->len, size);
+    list->len = clear_flag ? 0 : MIN (list->len, size);
 
     return TRUE;
 }
@@ -372,7 +354,7 @@ sort_vers (file_entry_t * a, file_entry_t * b)
 
     if (ad == bd || panels_options.mix_all_files)
     {
-        return str_verscmp (a->fname, b->fname) * reverse;
+        return filevercmp (a->fname, b->fname) * reverse;
     }
     else
     {
@@ -398,7 +380,7 @@ sort_ext (file_entry_t * a, file_entry_t * b)
             b->second_sort_key = str_create_key (extension (b->fname), case_sensitive);
 
         r = str_key_collate (a->second_sort_key, b->second_sort_key, case_sensitive);
-        if (r)
+        if (r != 0)
             return r * reverse;
         else
             return sort_name (a, b);
@@ -508,7 +490,7 @@ dir_list_sort (dir_list * list, GCompareFunc sort, const dir_sort_options_t * so
     file_entry_t *fentry;
     int dot_dot_found = 0;
 
-    if (list->len < 2)
+    if (list->len < 2 || sort == (GCompareFunc) unsorted)
         return;
 
     /* If there is an ".." entry the caller must take care to
@@ -545,6 +527,26 @@ dir_list_clean (dir_list * list)
     dir_list_grow (list, DIR_LIST_MIN_SIZE - list->size);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+void
+dir_list_free_list (dir_list * list)
+{
+    int i;
+
+    for (i = 0; i < list->len; i++)
+    {
+        file_entry_t *fentry;
+
+        fentry = &list->list[i];
+        g_free (fentry->fname);
+    }
+
+    MC_PTR_FREE (list->list);
+    list->len = 0;
+    list->size = 0;
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /** Used to set up a directory list when there is no access to a directory */
 
@@ -561,7 +563,7 @@ dir_list_init (dir_list * list)
     }
 
     fentry = &list->list[0];
-    memset (fentry, 0, sizeof (file_entry_t));
+    memset (fentry, 0, sizeof (*fentry));
     fentry->fnamelen = 2;
     fentry->fname = g_strndup ("..", fentry->fnamelen);
     fentry->f.link_to_dir = 0;
@@ -629,6 +631,7 @@ dir_list_load (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
     int link_to_dir, stale_link;
     struct stat st;
     file_entry_t *fentry;
+    const char *vpath_str;
 
     /* ".." (if any) must be the first entry in the list */
     if (!dir_list_init (list))
@@ -647,14 +650,10 @@ dir_list_load (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
 
     tree_store_start_check (vpath);
 
-    {
-        const char *vpath_str;
-
-        vpath_str = vfs_path_as_str (vpath);
-        /* Do not add a ".." entry to the root directory */
-        if (IS_PATH_SEP (vpath_str[0]) && vpath_str[1] == '\0')
-            list->len--;
-    }
+    vpath_str = vfs_path_as_str (vpath);
+    /* Do not add a ".." entry to the root directory */
+    if (IS_PATH_SEP (vpath_str[0]) && vpath_str[1] == '\0')
+        dir_list_clean (list);
 
     while ((dp = mc_readdir (dirp)) != NULL)
     {
@@ -724,7 +723,7 @@ dir_list_reload (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
         dfentry = &dir_copy.list[i];
 
         dfentry->fnamelen = fentry->fnamelen;
-        dfentry->fname = fentry->fname;
+        dfentry->fname = g_strndup (fentry->fname, fentry->fnamelen);
         dfentry->f.marked = fentry->f.marked;
         dfentry->f.dir_size_computed = fentry->f.dir_size_computed;
         dfentry->f.link_to_dir = fentry->f.link_to_dir;
@@ -738,6 +737,9 @@ dir_list_reload (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
         }
     }
 
+    /* save len for later dir_list_clean() */
+    dir_copy.len = list->len;
+
     /* Add ".." except to the root directory. The ".." entry
        (if any) must be the first in the list. */
     tmp_path = vfs_path_get_by_index (vpath, 0)->path;
@@ -748,9 +750,10 @@ dir_list_reload (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
     }
     else
     {
+        dir_list_clean (list);
         if (!dir_list_init (list))
         {
-            dir_list_clean (&dir_copy);
+            dir_list_free_list (&dir_copy);
             return;
         }
 
@@ -811,7 +814,7 @@ dir_list_reload (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
 
     dir_list_sort (list, sort, sort_op);
 
-    dir_list_clean (&dir_copy);
+    dir_list_free_list (&dir_copy);
     rotate_dash (FALSE);
 }
 
