@@ -1,7 +1,7 @@
 /*
    Panel layout module for the Midnight Commander
 
-   Copyright (C) 1995-2016
+   Copyright (C) 1995-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -82,7 +82,7 @@ panels_layout_t panels_layout = {
 };
 
 /* Controls the display of the rotating dash on the verbose mode */
-int nice_rotating_dash = 1;
+gboolean nice_rotating_dash = TRUE;
 
 /* The number of output lines shown (if available) */
 int output_lines = 0;
@@ -106,7 +106,7 @@ int ok_to_refresh = 1;
 
 /*** file scope macro definitions ****************************************************************/
 
-/* The maximum number of views managed by the set_display_type routine */
+/* The maximum number of views managed by the create_panel routine */
 /* Must be at least two (for current and other).  Please note that until */
 /* Janne gets around this, we will only manage two of them :-) */
 #define MAX_VIEWS 2
@@ -228,7 +228,7 @@ update_split (const WDialog * h)
         check_options[0].widget->state = panels_layout.vertical_equal ? 1 : 0;
     widget_redraw (WIDGET (check_options[0].widget));
 
-    tty_setcolor ((check_options[0].widget->state & C_BOOL) ? DISABLED_COLOR : COLOR_NORMAL);
+    tty_setcolor (check_options[0].widget->state ? DISABLED_COLOR : COLOR_NORMAL);
 
     widget_move (h, 6, 5);
     if (panels_layout.horizontal_split)
@@ -333,10 +333,10 @@ layout_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *
         {
             int _menubar_visible, _command_prompt, _keybar_visible, _message_visible;
 
-            _menubar_visible = check_options[1].widget->state & C_BOOL;
-            _command_prompt = check_options[2].widget->state & C_BOOL;
-            _keybar_visible = check_options[3].widget->state & C_BOOL;
-            _message_visible = check_options[4].widget->state & C_BOOL;
+            _menubar_visible = check_options[1].widget->state;
+            _command_prompt = check_options[2].widget->state;
+            _keybar_visible = check_options[3].widget->state;
+            _message_visible = check_options[4].widget->state;
 
             if (mc_global.tty.console_flag != '\0')
             {
@@ -370,7 +370,7 @@ layout_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *
         return MSG_HANDLED;
 
     case MSG_NOTIFY:
-        if (sender == WIDGET (radio_widget) && parm == (int) MSG_FOCUS)
+        if (sender == WIDGET (radio_widget))
         {
             if (panels_layout.horizontal_split != radio_widget->sel)
             {
@@ -404,18 +404,18 @@ layout_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *
             return MSG_HANDLED;
         }
 
-        if (sender == WIDGET (check_options[0].widget) && parm == (int) MSG_FOCUS)
+        if (sender == WIDGET (check_options[0].widget))
         {
             int eq;
 
             if (panels_layout.horizontal_split)
             {
-                panels_layout.horizontal_equal = check_options[0].widget->state & C_BOOL;
+                panels_layout.horizontal_equal = check_options[0].widget->state;
                 eq = panels_layout.horizontal_equal;
             }
             else
             {
-                panels_layout.vertical_equal = check_options[0].widget->state & C_BOOL;
+                panels_layout.vertical_equal = check_options[0].widget->state;
                 eq = panels_layout.vertical_equal;
             }
 
@@ -439,7 +439,7 @@ layout_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *
 /* --------------------------------------------------------------------------------------------- */
 
 static WDialog *
-init_layout (void)
+layout_dlg_create (void)
 {
     WDialog *layout_dlg;
     int l1 = 0, width;
@@ -513,7 +513,7 @@ init_layout (void)
         dlg_create (TRUE, 0, 0, 15, width, WPOS_CENTER, FALSE, dialog_colors, layout_callback, NULL,
                     "[Layout]", _("Layout"));
 
-#define XTRACT(i) *check_options[i].variable, check_options[i].text
+#define XTRACT(i) (*check_options[i].variable != 0), check_options[i].text
 
     /* "Panel split" groupbox */
     add_widget (layout_dlg, groupbox_new (2, 3, 6, l1, title1));
@@ -587,7 +587,7 @@ init_layout (void)
 static void
 panel_do_cols (int idx)
 {
-    if (get_display_type (idx) == view_listing)
+    if (get_panel_type (idx) == view_listing)
         set_panel_formats (PANEL (panels[idx].widget));
     else
         panel_update_cols (panels[idx].widget, frame_half);
@@ -597,23 +597,23 @@ panel_do_cols (int idx)
 /** Save current list_view widget directory into panel */
 
 static Widget *
-restore_into_right_dir_panel (int idx, Widget * from_widget)
+restore_into_right_dir_panel (int idx, gboolean last_was_panel, int y, int x, int lines, int cols)
 {
     WPanel *new_widget;
-    const char *saved_dir = panels[idx].last_saved_dir;
-    gboolean last_was_panel = (from_widget && get_display_type (idx) != view_listing);
-    const char *p_name = get_nth_panel_name (idx);
+    const char *p_name;
+
+    p_name = get_nth_panel_name (idx);
 
     if (last_was_panel)
     {
         vfs_path_t *saved_dir_vpath;
 
-        saved_dir_vpath = vfs_path_from_str (saved_dir);
-        new_widget = panel_new_with_dir (p_name, saved_dir_vpath);
+        saved_dir_vpath = vfs_path_from_str (panels[idx].last_saved_dir);
+        new_widget = panel_sized_with_dir_new (p_name, y, x, lines, cols, saved_dir_vpath);
         vfs_path_free (saved_dir_vpath);
     }
     else
-        new_widget = panel_new (p_name);
+        new_widget = panel_sized_new (p_name, y, x, lines, cols);
 
     return WIDGET (new_widget);
 }
@@ -642,7 +642,7 @@ layout_box (void)
 
     old_layout = panels_layout;
     old_output_lines = output_lines;
-    layout_dlg = init_layout ();
+    layout_dlg = layout_dlg_create ();
 
     if (dlg_run (layout_dlg) == B_ENTER)
     {
@@ -650,7 +650,7 @@ layout_box (void)
 
         for (i = 0; i < (size_t) LAYOUT_OPTIONS_COUNT; i++)
             if (check_options[i].widget != NULL)
-                *check_options[i].variable = check_options[i].widget->state & C_BOOL;
+                *check_options[i].variable = check_options[i].widget->state;
 
         output_lines = _output_lines;
     }
@@ -754,11 +754,10 @@ setup_panels (void)
     else
     {
         widget_set_size (panels[0].widget, start_y, 0, height, panels[0].widget->cols);
-        widget_set_size (panels[1].widget, start_y, panels_layout.left_panel_size, height,
+        widget_set_size (panels[1].widget, start_y, panels[1].widget->x, height,
                          panels[1].widget->cols);
     }
 
-
     widget_set_size (WIDGET (the_menubar), 0, 0, 1, COLS);
 
     if (command_prompt)
@@ -972,7 +971,7 @@ get_nth_panel_name (int num)
 /* since a lot of routines depend on the current_panel variable */
 
 void
-set_display_type (int num, panel_view_mode_t type)
+create_panel (int num, panel_view_mode_t type)
 {
     int x = 0, y = 0, cols = 0, lines = 0;
     unsigned int the_other = 0; /* Index to the other panel */
@@ -1033,9 +1032,13 @@ set_display_type (int num, panel_view_mode_t type)
     {
     case view_nothing:
     case view_listing:
-        new_widget = restore_into_right_dir_panel (num, old_widget);
-        widget_set_size (new_widget, y, x, lines, cols);
-        break;
+        {
+            gboolean last_was_panel;
+
+            last_was_panel = old_widget != NULL && get_panel_type (num) != view_listing;
+            new_widget = restore_into_right_dir_panel (num, last_was_panel, y, x, lines, cols);
+            break;
+        }
 
     case view_info:
         new_widget = WIDGET (info_new (y, x, lines, cols));
@@ -1234,7 +1237,7 @@ swap_panels (void)
 /* --------------------------------------------------------------------------------------------- */
 
 panel_view_mode_t
-get_display_type (int idx)
+get_panel_type (int idx)
 {
     return panels[idx].type;
 }
@@ -1304,7 +1307,7 @@ get_other_type (void)
 void
 save_panel_dir (int idx)
 {
-    panel_view_mode_t type = get_display_type (idx);
+    panel_view_mode_t type = get_panel_type (idx);
     Widget *widget = get_panel_widget (idx);
 
     if ((type == view_listing) && (widget != NULL))
@@ -1333,7 +1336,7 @@ get_panel_dir_for (const WPanel * widget)
     if (i >= MAX_VIEWS)
         return g_strdup (".");
 
-    if (get_display_type (i) == view_listing)
+    if (get_panel_type (i) == view_listing)
     {
         vfs_path_t *cwd_vpath;
 
