@@ -1,7 +1,7 @@
 /*
    Default values for keybinding engine
 
-   Copyright (C) 2009-2015
+   Copyright (C) 2009-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -38,6 +38,7 @@ GArray *main_keymap = NULL;
 GArray *main_x_keymap = NULL;
 GArray *panel_keymap = NULL;
 GArray *dialog_keymap = NULL;
+GArray *menu_keymap = NULL;
 GArray *input_keymap = NULL;
 GArray *listbox_keymap = NULL;
 GArray *tree_keymap = NULL;
@@ -55,6 +56,7 @@ GArray *diff_keymap = NULL;
 const global_keymap_t *main_map = NULL;
 const global_keymap_t *main_x_map = NULL;
 const global_keymap_t *panel_map = NULL;
+const global_keymap_t *menu_map = NULL;
 const global_keymap_t *tree_map = NULL;
 const global_keymap_t *help_map = NULL;
 
@@ -83,6 +85,7 @@ typedef struct global_keymap_ini_t
 
 /* midnight */
 static const global_keymap_ini_t default_main_keymap[] = {
+    {"ChangePanel", "tab"},
     {"Help", "f1"},
     {"UserMenu", "f2"},
     {"View", "f3"},
@@ -100,9 +103,11 @@ static const global_keymap_ini_t default_main_keymap[] = {
     /* Copy useful information to the command line */
     {"PutCurrentPath", "alt-a"},
     {"PutOtherPath", "alt-shift-a"},
+    {"PutCurrentSelected", "alt-enter; ctrl-enter"},
+    {"PutCurrentFullSelected", "ctrl-shift-enter"},
     {"CdQuick", "alt-c"},
     /* To access the directory hotlist */
-    {"HotList", "ctrl-backspace"},
+    {"HotList", "ctrl-backslash"},
     /* Suspend */
     {"Suspend", "ctrl-z"},
     /* The filtered view command */
@@ -112,7 +117,6 @@ static const global_keymap_ini_t default_main_keymap[] = {
     /* Panel refresh */
     {"Reread", "ctrl-r"},
     /* Switch listing between long, user defined and full formats */
-    {"PanelListingSwitch", "alt-t"},
     /* Swap panels */
     {"Swap", "ctrl-u"},
     /* Resize panels */
@@ -165,6 +169,7 @@ static const global_keymap_ini_t default_main_x_keymap[] = {
 
 /* panel */
 static const global_keymap_ini_t default_panel_keymap[] = {
+    {"CycleListingFormat", "alt-t"},
     {"PanelOtherCd", "alt-o"},
     {"PanelOtherCdLink", "alt-l"},
     {"CopySingle", "f15"},
@@ -222,6 +227,20 @@ static const global_keymap_ini_t default_dialog_keymap[] = {
     {NULL, NULL}
 };
 
+/* menubar */
+static const global_keymap_ini_t default_menu_keymap[] = {
+    {"Help", "f1"},
+    {"Left", "left; ctrl-b"},
+    {"Right", "right; ctrl-f"},
+    {"Up", "up; ctrl-p"},
+    {"Down", "down; enter; ctrl-n"},
+    {"Home", "home; alt-lt; ctrl-a"},
+    {"End", "end; alt-gt; ctrl-e"},
+    {"Enter", "enter"},
+    {"Quit", "f10; ctrl-g; esc"},
+    {NULL, NULL}
+};
+
 /* input line */
 static const global_keymap_ini_t default_input_keymap[] = {
     /* Motion */
@@ -272,12 +291,12 @@ static const global_keymap_ini_t default_listbox_keymap[] = {
 
 /* tree */
 static const global_keymap_ini_t default_tree_keymap[] = {
-    {"Help = f1"},
-    {"Rescan = f2; ctrl-r"},
-    {"Forget = f3"},
-    {"ToggleNavigation = f4"},
-    {"Copy = f5"},
-    {"Move = f6"},
+    {"Help", "f1"},
+    {"Rescan", "f2; ctrl-r"},
+    {"Forget", "f3"},
+    {"ToggleNavigation", "f4"},
+    {"Copy", "f5"},
+    {"Move", "f6"},
 #if 0
     {"MakeDir", "f7"},
 #endif
@@ -466,6 +485,7 @@ static const global_keymap_ini_t default_viewer_keymap[] = {
     {"SearchBackward", "question"},
     {"SearchForwardContinue", "ctrl-s"},
     {"SearchBackwardContinue", "ctrl-r"},
+    {"SearchOppositeContinue", "shift-n"},
     {NULL, NULL}
 };
 
@@ -500,6 +520,7 @@ static const global_keymap_ini_t default_viewer_hex_keymap[] = {
     {"SearchBackward", "question"},
     {"SearchForwardContinue", "ctrl-s"},
     {"SearchBackwardContinue", "ctrl-r"},
+    {"SearchOppositeContinue", "shift-n"},
     {NULL, NULL}
 };
 
@@ -585,6 +606,7 @@ create_default_keymap (void)
     create_default_keymap_section (keymap, KEYMAP_SECTION_MAIN_EXT, default_main_x_keymap);
     create_default_keymap_section (keymap, KEYMAP_SECTION_PANEL, default_panel_keymap);
     create_default_keymap_section (keymap, KEYMAP_SECTION_DIALOG, default_dialog_keymap);
+    create_default_keymap_section (keymap, KEYMAP_SECTION_MENU, default_menu_keymap);
     create_default_keymap_section (keymap, KEYMAP_SECTION_INPUT, default_input_keymap);
     create_default_keymap_section (keymap, KEYMAP_SECTION_LISTBOX, default_listbox_keymap);
     create_default_keymap_section (keymap, KEYMAP_SECTION_TREE, default_tree_keymap);
