@@ -2,7 +2,7 @@
    Virtual File System: FISH implementation for transfering files over
    shell connections.
 
-   Copyright (C) 1998-2018
+   Copyright (C) 1998-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -117,12 +117,15 @@ int fish_directory_timeout = 900;
 #define FISH_HAVE_DATE_MDYT   32
 #define FISH_HAVE_TAIL        64
 
-#define SUP ((fish_super_data_t *) super->data)
+#define FISH_SUPER(super) ((fish_super_t *) (super))
+#define FISH_FILE_HANDLER(fh) ((fish_file_handler_t *) fh)
 
 /*** file scope type declarations ****************************************************************/
 
 typedef struct
 {
+    struct vfs_s_super base;    /* base class */
+
     int sockr;
     int sockw;
     char *scr_ls;
@@ -142,20 +145,23 @@ typedef struct
     char *scr_info;
     int host_flags;
     char *scr_env;
-} fish_super_data_t;
+} fish_super_t;
 
 typedef struct
 {
+    vfs_file_handler_t base;    /* base class */
+
     off_t got;
     off_t total;
     gboolean append;
-} fish_fh_data_t;
+} fish_file_handler_t;
 
 /*** file scope variables ************************************************************************/
 
 static char reply_str[80];
 
-static struct vfs_class vfs_fish_ops;
+static struct vfs_s_subclass fish_subclass;
+static struct vfs_class *vfs_fish_ops = VFS_CLASS (&fish_subclass);
 
 /* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
@@ -266,7 +272,7 @@ fish_command (struct vfs_class *me, struct vfs_s_super *super, int wait_reply, c
               size_t cmd_len)
 {
     ssize_t status;
-    FILE *logfile = MEDATA->logfile;
+    FILE *logfile = me->logfile;
 
     if (cmd_len == (size_t) (-1))
         cmd_len = strlen (cmd);
@@ -281,14 +287,14 @@ fish_command (struct vfs_class *me, struct vfs_s_super *super, int wait_reply, c
     }
 
     tty_enable_interrupt_key ();
-    status = write (SUP->sockw, cmd, cmd_len);
+    status = write (FISH_SUPER (super)->sockw, cmd, cmd_len);
     tty_disable_interrupt_key ();
 
     if (status < 0)
         return TRANSIENT;
 
     if (wait_reply)
-        return fish_get_reply (me, SUP->sockr,
+        return fish_get_reply (me, FISH_SUPER (super)->sockr,
                                (wait_reply & WANT_STRING) != 0 ? reply_str :
                                NULL, sizeof (reply_str) - 1);
     return COMPLETE;
@@ -304,7 +310,7 @@ fish_command_va (struct vfs_class *me, struct vfs_s_super *super, int wait_reply
     int r;
     GString *command;
 
-    command = g_string_new (SUP->scr_env);
+    command = g_string_new (FISH_SUPER (super)->scr_env);
     g_string_append_vprintf (command, vars, ap);
     g_string_append (command, scr);
     r = fish_command (me, super, wait_reply, command->str, command->len);
@@ -343,7 +349,7 @@ fish_send_command (struct vfs_class *me, struct vfs_s_super *super, int flags, c
     va_start (ap, vars);
     r = fish_command_va (me, super, WAIT_REPLY, scr, vars, ap);
     va_end (ap);
-    vfs_stamp_create (&vfs_fish_ops, super);
+    vfs_stamp_create (vfs_fish_ops, super);
 
     if (r != COMPLETE)
         ERRNOR (E_REMOTE, -1);
@@ -355,35 +361,48 @@ fish_send_command (struct vfs_class *me, struct vfs_s_super *super, int flags, c
 
 /* --------------------------------------------------------------------------------------------- */
 
+static struct vfs_s_super *
+fish_new_archive (struct vfs_class *me)
+{
+    fish_super_t *arch;
+
+    arch = g_new0 (fish_super_t, 1);
+    arch->base.me = me;
+
+    return VFS_SUPER (arch);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 fish_free_archive (struct vfs_class *me, struct vfs_s_super *super)
 {
-    if ((SUP->sockw != -1) || (SUP->sockr != -1))
+    fish_super_t *fish_super = FISH_SUPER (super);
+
+    if ((fish_super->sockw != -1) || (fish_super->sockr != -1))
     {
         vfs_print_message (_("fish: Disconnecting from %s"), super->name ? super->name : "???");
         fish_command (me, super, NONE, "#BYE\nexit\n", -1);
-        close (SUP->sockw);
-        close (SUP->sockr);
-        SUP->sockw = SUP->sockr = -1;
+        close (fish_super->sockw);
+        close (fish_super->sockr);
+        fish_super->sockw = fish_super->sockr = -1;
     }
-    g_free (SUP->scr_ls);
-    g_free (SUP->scr_exists);
-    g_free (SUP->scr_mkdir);
-    g_free (SUP->scr_unlink);
-    g_free (SUP->scr_chown);
-    g_free (SUP->scr_chmod);
-    g_free (SUP->scr_utime);
-    g_free (SUP->scr_rmdir);
-    g_free (SUP->scr_ln);
-    g_free (SUP->scr_mv);
-    g_free (SUP->scr_hardlink);
-    g_free (SUP->scr_get);
-    g_free (SUP->scr_send);
-    g_free (SUP->scr_append);
-    g_free (SUP->scr_info);
-    g_free (SUP->scr_env);
-    g_free (SUP);
-    super->data = NULL;
+    g_free (fish_super->scr_ls);
+    g_free (fish_super->scr_exists);
+    g_free (fish_super->scr_mkdir);
+    g_free (fish_super->scr_unlink);
+    g_free (fish_super->scr_chown);
+    g_free (fish_super->scr_chmod);
+    g_free (fish_super->scr_utime);
+    g_free (fish_super->scr_rmdir);
+    g_free (fish_super->scr_ln);
+    g_free (fish_super->scr_mv);
+    g_free (fish_super->scr_hardlink);
+    g_free (fish_super->scr_get);
+    g_free (fish_super->scr_send);
+    g_free (fish_super->scr_append);
+    g_free (fish_super->scr_info);
+    g_free (fish_super->scr_env);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -405,9 +424,9 @@ fish_pipeopen (struct vfs_s_super *super, const char *path, const char *argv[])
             vfs_die ("Cannot fork(): %m.");
         /* We are the parent */
         close (fileset1[0]);
-        SUP->sockw = fileset1[1];
+        FISH_SUPER (super)->sockw = fileset1[1];
         close (fileset2[1]);
-        SUP->sockr = fileset2[0];
+        FISH_SUPER (super)->sockr = fileset2[0];
     }
     else
     {
@@ -464,19 +483,21 @@ fish_set_env (int flags)
 static gboolean
 fish_info (struct vfs_class *me, struct vfs_s_super *super)
 {
-    if (fish_command (me, super, NONE, SUP->scr_info, -1) == COMPLETE)
+    fish_super_t *fish_super = FISH_SUPER (super);
+
+    if (fish_command (me, super, NONE, fish_super->scr_info, -1) == COMPLETE)
     {
         while (TRUE)
         {
             int res;
             char buffer[BUF_8K];
 
-            res = vfs_s_get_line_interruptible (me, buffer, sizeof (buffer), SUP->sockr);
+            res = vfs_s_get_line_interruptible (me, buffer, sizeof (buffer), fish_super->sockr);
             if ((res == 0) || (res == EINTR))
                 ERRNOR (ECONNRESET, FALSE);
             if (strncmp (buffer, "### ", 4) == 0)
                 break;
-            SUP->host_flags = atol (buffer);
+            fish_super->host_flags = atol (buffer);
         }
         return TRUE;
     }
@@ -534,11 +555,12 @@ fish_open_archive_pipeopen (struct vfs_s_super *super)
 static gboolean
 fish_open_archive_talk (struct vfs_class *me, struct vfs_s_super *super)
 {
+    fish_super_t *fish_super = FISH_SUPER (super);
     char answer[2048];
 
     printf ("\n%s\n", _("fish: Waiting for initial line..."));
 
-    if (vfs_s_get_line (me, SUP->sockr, answer, sizeof (answer), ':') == 0)
+    if (vfs_s_get_line (me, fish_super->sockr, answer, sizeof (answer), ':') == 0)
         return FALSE;
 
     if (strstr (answer, "assword") != NULL)
@@ -568,8 +590,8 @@ fish_open_archive_talk (struct vfs_class *me, struct vfs_s_super *super)
             size_t str_len;
 
             str_len = strlen (super->path_element->password);
-            if ((write (SUP.sockw, super->path_element->password, str_len) != (ssize_t) str_len)
-                || (write (SUP->sockw, "\n", 1) != 1))
+            if ((write (fish_super.sockw, super->path_element->password, str_len) !=
+                 (ssize_t) str_len) || (write (fish_super->sockw, "\n", 1) != 1))
                 return FALSE;
         }
 #endif
@@ -623,7 +645,7 @@ fish_open_archive_int (struct vfs_class *me, struct vfs_s_super *super)
 
     vfs_print_message ("%s", _("fish: Getting host info..."));
     if (fish_info (me, super))
-        SUP->scr_env = fish_set_env (SUP->host_flags);
+        FISH_SUPER (super)->scr_env = fish_set_env (FISH_SUPER (super)->host_flags);
 
 #if 0
     super->name =
@@ -644,53 +666,54 @@ static int
 fish_open_archive (struct vfs_s_super *super,
                    const vfs_path_t * vpath, const vfs_path_element_t * vpath_element)
 {
+    fish_super_t *fish_super = FISH_SUPER (super);
+
     (void) vpath;
 
-    super->data = g_new0 (fish_super_data_t, 1);
     super->path_element = vfs_path_element_clone (vpath_element);
 
     if (strncmp (vpath_element->vfs_prefix, "rsh", 3) == 0)
         super->path_element->port = FISH_FLAG_RSH;
 
-    SUP->scr_ls =
+    fish_super->scr_ls =
         fish_load_script_from_file (super->path_element->host, FISH_LS_FILE, FISH_LS_DEF_CONTENT);
-    SUP->scr_exists =
+    fish_super->scr_exists =
         fish_load_script_from_file (super->path_element->host, FISH_EXISTS_FILE,
                                     FISH_EXISTS_DEF_CONTENT);
-    SUP->scr_mkdir =
+    fish_super->scr_mkdir =
         fish_load_script_from_file (super->path_element->host, FISH_MKDIR_FILE,
                                     FISH_MKDIR_DEF_CONTENT);
-    SUP->scr_unlink =
+    fish_super->scr_unlink =
         fish_load_script_from_file (super->path_element->host, FISH_UNLINK_FILE,
                                     FISH_UNLINK_DEF_CONTENT);
-    SUP->scr_chown =
+    fish_super->scr_chown =
         fish_load_script_from_file (super->path_element->host, FISH_CHOWN_FILE,
                                     FISH_CHOWN_DEF_CONTENT);
-    SUP->scr_chmod =
+    fish_super->scr_chmod =
         fish_load_script_from_file (super->path_element->host, FISH_CHMOD_FILE,
                                     FISH_CHMOD_DEF_CONTENT);
-    SUP->scr_utime =
+    fish_super->scr_utime =
         fish_load_script_from_file (super->path_element->host, FISH_UTIME_FILE,
                                     FISH_UTIME_DEF_CONTENT);
-    SUP->scr_rmdir =
+    fish_super->scr_rmdir =
         fish_load_script_from_file (super->path_element->host, FISH_RMDIR_FILE,
                                     FISH_RMDIR_DEF_CONTENT);
-    SUP->scr_ln =
+    fish_super->scr_ln =
         fish_load_script_from_file (super->path_element->host, FISH_LN_FILE, FISH_LN_DEF_CONTENT);
-    SUP->scr_mv =
+    fish_super->scr_mv =
         fish_load_script_from_file (super->path_element->host, FISH_MV_FILE, FISH_MV_DEF_CONTENT);
-    SUP->scr_hardlink =
+    fish_super->scr_hardlink =
         fish_load_script_from_file (super->path_element->host, FISH_HARDLINK_FILE,
                                     FISH_HARDLINK_DEF_CONTENT);
-    SUP->scr_get =
+    fish_super->scr_get =
         fish_load_script_from_file (super->path_element->host, FISH_GET_FILE, FISH_GET_DEF_CONTENT);
-    SUP->scr_send =
+    fish_super->scr_send =
         fish_load_script_from_file (super->path_element->host, FISH_SEND_FILE,
                                     FISH_SEND_DEF_CONTENT);
-    SUP->scr_append =
+    fish_super->scr_append =
         fish_load_script_from_file (super->path_element->host, FISH_APPEND_FILE,
                                     FISH_APPEND_DEF_CONTENT);
-    SUP->scr_info =
+    fish_super->scr_info =
         fish_load_script_from_file (super->path_element->host, FISH_INFO_FILE,
                                     FISH_INFO_DEF_CONTENT);
 
@@ -731,7 +754,6 @@ fish_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path)
     struct vfs_s_super *super = dir->super;
     char buffer[BUF_8K] = "\0";
     struct vfs_s_entry *ent = NULL;
-    FILE *logfile;
     char *quoted_path;
     int reply_code;
 
@@ -739,10 +761,9 @@ fish_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path)
      * Simple FISH debug interface :]
      */
 #if 0
-    if (MEDATA->logfile == NULL)
-        MEDATA->logfile = fopen ("/tmp/mc-FISH.sh", "w");
+    if (me->logfile == NULL)
+        me->logfile = fopen ("/tmp/mc-FISH.sh", "w");
 #endif
-    logfile = MEDATA->logfile;
 
     vfs_print_message (_("fish: Reading directory %s..."), remote_path);
 
@@ -750,7 +771,8 @@ fish_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path)
     dir->timestamp.tv_sec += fish_directory_timeout;
 
     quoted_path = strutils_shell_escape (remote_path);
-    (void) fish_command_v (me, super, NONE, SUP->scr_ls, "FISH_FILENAME=%s;\n", quoted_path);
+    (void) fish_command_v (me, super, NONE, FISH_SUPER (super)->scr_ls, "FISH_FILENAME=%s;\n",
+                           quoted_path);
     g_free (quoted_path);
 
     ent = vfs_s_generate_entry (me, NULL, dir, 0);
@@ -759,7 +781,7 @@ fish_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path)
     {
         int res;
 
-        res = vfs_s_get_line_interruptible (me, buffer, sizeof (buffer), SUP->sockr);
+        res = vfs_s_get_line_interruptible (me, buffer, sizeof (buffer), FISH_SUPER (super)->sockr);
 
         if ((res == 0) || (res == EINTR))
         {
@@ -767,11 +789,11 @@ fish_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path)
             me->verrno = ECONNRESET;
             goto error;
         }
-        if (logfile != NULL)
+        if (me->logfile != NULL)
         {
-            fputs (buffer, logfile);
-            fputs ("\n", logfile);
-            fflush (logfile);
+            fputs (buffer, me->logfile);
+            fputs ("\n", me->logfile);
+            fflush (me->logfile);
         }
         if (strncmp (buffer, "### ", 4) == 0)
             break;
@@ -944,8 +966,9 @@ fish_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path)
 static int
 fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char *localname)
 {
-    fish_fh_data_t *fish = (fish_fh_data_t *) fh->data;
-    struct vfs_s_super *super = FH_SUPER;
+    fish_file_handler_t *fish = FISH_FILE_HANDLER (fh);
+    struct vfs_s_super *super = VFS_FILE_HANDLER_SUPER (fh);
+    fish_super_t *fish_super = FISH_SUPER (super);
     int code;
     off_t total = 0;
     char buffer[BUF_8K];
@@ -995,9 +1018,11 @@ fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char
     vfs_print_message (_("fish: store %s: sending command..."), quoted_name);
 
     /* FIXME: File size is limited to ULONG_MAX */
-    code = fish_command_v (me, super, WAIT_REPLY, fish->append ? SUP->scr_append : SUP->scr_send,
-                           "FISH_FILENAME=%s FISH_FILESIZE=%" PRIuMAX ";\n", quoted_name,
-                           (uintmax_t) s.st_size);
+    code =
+        fish_command_v (me, super, WAIT_REPLY,
+                        fish->append ? fish_super->scr_append : fish_super->scr_send,
+                        "FISH_FILENAME=%s FISH_FILESIZE=%" PRIuMAX ";\n", quoted_name,
+                        (uintmax_t) s.st_size);
     g_free (quoted_name);
 
     if (code != PRELIM)
@@ -1022,7 +1047,7 @@ fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char
         if (n == 0)
             break;
 
-        t = write (SUP->sockw, buffer, n);
+        t = write (fish_super->sockw, buffer, n);
         if (t != n)
         {
             if (t == -1)
@@ -1038,13 +1063,13 @@ fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char
     }
     close (h);
 
-    if (fish_get_reply (me, SUP->sockr, NULL, 0) != COMPLETE)
+    if (fish_get_reply (me, fish_super->sockr, NULL, 0) != COMPLETE)
         ERRNOR (E_REMOTE, -1);
     return 0;
 
   error_return:
     close (h);
-    fish_get_reply (me, SUP->sockr, NULL, 0);
+    fish_get_reply (me, fish_super->sockr, NULL, 0);
     return -1;
 }
 
@@ -1053,16 +1078,11 @@ fish_file_store (struct vfs_class *me, vfs_file_handler_t * fh, char *name, char
 static int
 fish_linear_start (struct vfs_class *me, vfs_file_handler_t * fh, off_t offset)
 {
-    fish_fh_data_t *fish;
-    struct vfs_s_super *super = FH_SUPER;
+    fish_file_handler_t *fish = FISH_FILE_HANDLER (fh);
+    struct vfs_s_super *super = VFS_FILE_HANDLER_SUPER (fh);
     char *name;
     char *quoted_name;
 
-    if (fh->data == NULL)
-        fh->data = g_new0 (fish_fh_data_t, 1);
-
-    fish = (fish_fh_data_t *) fh->data;
-
     name = vfs_s_fullpath (me, fh->ino);
     if (name == NULL)
         return 0;
@@ -1078,7 +1098,7 @@ fish_linear_start (struct vfs_class *me, vfs_file_handler_t * fh, off_t offset)
      */
 
     offset =
-        fish_command_v (me, super, WANT_STRING, SUP->scr_get,
+        fish_command_v (me, super, WANT_STRING, FISH_SUPER (super)->scr_get,
                         "FISH_FILENAME=%s FISH_START_OFFSET=%" PRIuMAX ";\n", quoted_name,
                         (uintmax_t) offset);
     g_free (quoted_name);
@@ -1103,8 +1123,8 @@ fish_linear_start (struct vfs_class *me, vfs_file_handler_t * fh, off_t offset)
 static void
 fish_linear_abort (struct vfs_class *me, vfs_file_handler_t * fh)
 {
-    fish_fh_data_t *fish = (fish_fh_data_t *) fh->data;
-    struct vfs_s_super *super = FH_SUPER;
+    fish_file_handler_t *fish = FISH_FILE_HANDLER (fh);
+    struct vfs_s_super *super = VFS_FILE_HANDLER_SUPER (fh);
     char buffer[BUF_8K];
     ssize_t n;
 
@@ -1115,7 +1135,7 @@ fish_linear_abort (struct vfs_class *me, vfs_file_handler_t * fh)
         n = MIN ((off_t) sizeof (buffer), (fish->total - fish->got));
         if (n != 0)
         {
-            n = read (SUP->sockr, buffer, n);
+            n = read (FISH_SUPER (super)->sockr, buffer, n);
             if (n < 0)
                 return;
             fish->got += n;
@@ -1123,7 +1143,7 @@ fish_linear_abort (struct vfs_class *me, vfs_file_handler_t * fh)
     }
     while (n != 0);
 
-    if (fish_get_reply (me, SUP->sockr, NULL, 0) != COMPLETE)
+    if (fish_get_reply (me, FISH_SUPER (super)->sockr, NULL, 0) != COMPLETE)
         vfs_print_message ("%s", _("Error reported after abort."));
     else
         vfs_print_message ("%s", _("Aborted transfer would be successful."));
@@ -1134,13 +1154,13 @@ fish_linear_abort (struct vfs_class *me, vfs_file_handler_t * fh)
 static ssize_t
 fish_linear_read (struct vfs_class *me, vfs_file_handler_t * fh, void *buf, size_t len)
 {
-    fish_fh_data_t *fish = (fish_fh_data_t *) fh->data;
-    struct vfs_s_super *super = FH_SUPER;
+    fish_file_handler_t *fish = FISH_FILE_HANDLER (fh);
+    struct vfs_s_super *super = VFS_FILE_HANDLER_SUPER (fh);
     ssize_t n = 0;
 
     len = MIN ((size_t) (fish->total - fish->got), len);
     tty_disable_interrupt_key ();
-    while (len != 0 && ((n = read (SUP->sockr, buf, len)) < 0))
+    while (len != 0 && ((n = read (FISH_SUPER (super)->sockr, buf, len)) < 0))
     {
         if ((errno == EINTR) && !tty_got_interrupt ())
             continue;
@@ -1152,7 +1172,7 @@ fish_linear_read (struct vfs_class *me, vfs_file_handler_t * fh, void *buf, size
         fish->got += n;
     else if (n < 0)
         fish_linear_abort (me, fh);
-    else if (fish_get_reply (me, SUP->sockr, NULL, 0) != COMPLETE)
+    else if (fish_get_reply (me, FISH_SUPER (super)->sockr, NULL, 0) != COMPLETE)
         ERRNOR (E_REMOTE, -1);
     ERRNOR (errno, n);
 }
@@ -1162,7 +1182,7 @@ fish_linear_read (struct vfs_class *me, vfs_file_handler_t * fh, void *buf, size
 static void
 fish_linear_close (struct vfs_class *me, vfs_file_handler_t * fh)
 {
-    fish_fh_data_t *fish = (fish_fh_data_t *) fh->data;
+    fish_file_handler_t *fish = FISH_FILE_HANDLER (fh);
 
     if (fish->total != fish->got)
         fish_linear_abort (me, fh);
@@ -1184,14 +1204,15 @@ fish_ctl (void *fh, int ctlop, void *arg)
     {
     case VFS_CTL_IS_NOTREADY:
         {
+            vfs_file_handler_t *file = VFS_FILE_HANDLER (fh);
             int v;
 
-            if (FH->linear == LS_NOT_LINEAR)
+            if (file->linear == LS_NOT_LINEAR)
                 vfs_die ("You may not do this");
-            if (FH->linear == LS_LINEAR_CLOSED || FH->linear == LS_LINEAR_PREOPEN)
+            if (file->linear == LS_LINEAR_CLOSED || file->linear == LS_LINEAR_PREOPEN)
                 return 0;
 
-            v = vfs_s_select_on_two (FH_SUPER->u.fish.sockr, 0);
+            v = vfs_s_select_on_two (VFS_FILE_HANDLER_SUPER (fh)->u.fish.sockr, 0);
 
             return (((v < 0) && (errno == EINTR)) || v == 0) ? 1 : 0;
         }
@@ -1226,7 +1247,7 @@ fish_rename (const vfs_path_t * vpath1, const vfs_path_t * vpath2)
     rpath2 = strutils_shell_escape (crpath2);
 
     ret =
-        fish_send_command (path_element->class, super2, OPT_FLUSH, SUP->scr_mv,
+        fish_send_command (path_element->class, super2, OPT_FLUSH, FISH_SUPER (super)->scr_mv,
                            "FISH_FILEFROM=%s FISH_FILETO=%s;\n", rpath1, rpath2);
 
     g_free (rpath1);
@@ -1260,7 +1281,7 @@ fish_link (const vfs_path_t * vpath1, const vfs_path_t * vpath2)
     rpath2 = strutils_shell_escape (crpath2);
 
     ret =
-        fish_send_command (path_element->class, super2, OPT_FLUSH, SUP->scr_hardlink,
+        fish_send_command (path_element->class, super2, OPT_FLUSH, FISH_SUPER (super)->scr_hardlink,
                            "FISH_FILEFROM=%s FISH_FILETO=%s;\n", rpath1, rpath2);
 
     g_free (rpath1);
@@ -1291,7 +1312,7 @@ fish_symlink (const vfs_path_t * vpath1, const vfs_path_t * vpath2)
     qsetto = strutils_shell_escape (vfs_path_get_by_index (vpath1, -1)->path);
 
     ret =
-        fish_send_command (path_element->class, super, OPT_FLUSH, SUP->scr_ln,
+        fish_send_command (path_element->class, super, OPT_FLUSH, FISH_SUPER (super)->scr_ln,
                            "FISH_FILEFROM=%s FISH_FILETO=%s;\n", qsetto, rpath);
 
     g_free (qsetto);
@@ -1356,8 +1377,9 @@ fish_chmod (const vfs_path_t * vpath, mode_t mode)
     rpath = strutils_shell_escape (crpath);
 
     ret =
-        fish_send_command (path_element->class, super, OPT_FLUSH, SUP->scr_chmod,
-                           "FISH_FILENAME=%s FISH_FILEMODE=%4.4o;\n", rpath, (int) (mode & 07777));
+        fish_send_command (path_element->class, super, OPT_FLUSH, FISH_SUPER (super)->scr_chmod,
+                           "FISH_FILENAME=%s FISH_FILEMODE=%4.4o;\n", rpath,
+                           (unsigned int) (mode & 07777));
 
     g_free (rpath);
 
@@ -1399,7 +1421,7 @@ fish_chown (const vfs_path_t * vpath, uid_t owner, gid_t group)
 
     /* FIXME: what should we report if chgrp succeeds but chown fails? */
     ret =
-        fish_send_command (path_element->class, super, OPT_FLUSH, SUP->scr_chown,
+        fish_send_command (path_element->class, super, OPT_FLUSH, FISH_SUPER (super)->scr_chown,
                            "FISH_FILENAME=%s FISH_FILEOWNER=%s FISH_FILEGROUP=%s;\n", rpath, sowner,
                            sgroup);
 
@@ -1478,7 +1500,7 @@ fish_utime (const vfs_path_t * vpath, mc_timesbuf_t * times)
                 gmt->tm_year + 1900, gmt->tm_mon + 1, gmt->tm_mday,
                 gmt->tm_hour, gmt->tm_min, gmt->tm_sec, mtime_nsec);
 
-    ret = fish_send_command (path_element->class, super, OPT_FLUSH, SUP->scr_utime,
+    ret = fish_send_command (path_element->class, super, OPT_FLUSH, FISH_SUPER (super)->scr_utime,
                              "FISH_FILENAME=%s FISH_FILEATIME=%ld FISH_FILEMTIME=%ld "
                              "FISH_TOUCHATIME=%s FISH_TOUCHMTIME=%s FISH_TOUCHATIME_W_NSEC=\"%s\" "
                              "FISH_TOUCHMTIME_W_NSEC=\"%s\";\n", rpath, (long) atime, (long) mtime,
@@ -1509,7 +1531,7 @@ fish_unlink (const vfs_path_t * vpath)
     rpath = strutils_shell_escape (crpath);
 
     ret =
-        fish_send_command (path_element->class, super, OPT_FLUSH, SUP->scr_unlink,
+        fish_send_command (path_element->class, super, OPT_FLUSH, FISH_SUPER (super)->scr_unlink,
                            "FISH_FILENAME=%s;\n", rpath);
 
     g_free (rpath);
@@ -1537,7 +1559,7 @@ fish_exists (const vfs_path_t * vpath)
     rpath = strutils_shell_escape (crpath);
 
     ret =
-        fish_send_command (path_element->class, super, OPT_FLUSH, SUP->scr_exists,
+        fish_send_command (path_element->class, super, OPT_FLUSH, FISH_SUPER (super)->scr_exists,
                            "FISH_FILENAME=%s;\n", rpath);
 
     g_free (rpath);
@@ -1567,7 +1589,7 @@ fish_mkdir (const vfs_path_t * vpath, mode_t mode)
     rpath = strutils_shell_escape (crpath);
 
     ret =
-        fish_send_command (path_element->class, super, OPT_FLUSH, SUP->scr_mkdir,
+        fish_send_command (path_element->class, super, OPT_FLUSH, FISH_SUPER (super)->scr_mkdir,
                            "FISH_FILENAME=%s;\n", rpath);
     g_free (rpath);
 
@@ -1602,7 +1624,7 @@ fish_rmdir (const vfs_path_t * vpath)
     rpath = strutils_shell_escape (crpath);
 
     ret =
-        fish_send_command (path_element->class, super, OPT_FLUSH, SUP->scr_rmdir,
+        fish_send_command (path_element->class, super, OPT_FLUSH, FISH_SUPER (super)->scr_rmdir,
                            "FISH_FILENAME=%s;\n", rpath);
 
     g_free (rpath);
@@ -1612,11 +1634,15 @@ fish_rmdir (const vfs_path_t * vpath)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static void
-fish_fh_free_data (vfs_file_handler_t * fh)
+static vfs_file_handler_t *
+fish_fh_new (struct vfs_s_inode *ino, gboolean changed)
 {
-    if (fh != NULL)
-        MC_PTR_FREE (fh->data);
+    fish_file_handler_t *fh;
+
+    fh = g_new0 (fish_file_handler_t, 1);
+    vfs_s_init_fh (VFS_FILE_HANDLER (fh), ino, changed);
+
+    return VFS_FILE_HANDLER (fh);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1624,13 +1650,10 @@ fish_fh_free_data (vfs_file_handler_t * fh)
 static int
 fish_fh_open (struct vfs_class *me, vfs_file_handler_t * fh, int flags, mode_t mode)
 {
-    fish_fh_data_t *fish;
+    fish_file_handler_t *fish = FISH_FILE_HANDLER (fh);
 
     (void) mode;
 
-    fh->data = g_new0 (fish_fh_data_t, 1);
-    fish = (fish_fh_data_t *) fh->data;
-
     /* File will be written only, so no need to retrieve it */
     if (((flags & O_WRONLY) == O_WRONLY) && ((flags & (O_RDONLY | O_RDWR)) == 0))
     {
@@ -1662,7 +1685,6 @@ fish_fh_open (struct vfs_class *me, vfs_file_handler_t * fh, int flags, mode_t m
     return 0;
 
   fail:
-    fish_fh_free_data (fh);
     return -1;
 }
 
@@ -1673,7 +1695,7 @@ fish_fill_names (struct vfs_class *me, fill_names_f func)
 {
     GList *iter;
 
-    for (iter = MEDATA->supers; iter != NULL; iter = g_list_next (iter))
+    for (iter = VFS_SUBCLASS (me)->supers; iter != NULL; iter = g_list_next (iter))
     {
         const struct vfs_s_super *super = (const struct vfs_s_super *) iter->data;
 
@@ -1699,7 +1721,7 @@ fish_fill_names (struct vfs_class *me, fill_names_f func)
         }
 
         name =
-            g_strconcat (vfs_fish_ops.prefix, VFS_PATH_URL_DELIMITER,
+            g_strconcat (vfs_fish_ops->prefix, VFS_PATH_URL_DELIMITER,
                          super->path_element->user, "@", super->path_element->host, flags,
                          PATH_SEP_STR, super->path_element->path, (char *) NULL);
         func (name);
@@ -1725,43 +1747,38 @@ fish_open (const vfs_path_t * vpath, int flags, mode_t mode)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-init_fish (void)
+vfs_init_fish (void)
 {
-    static struct vfs_s_subclass fish_subclass;
-
     tcp_init ();
 
-    fish_subclass.flags = VFS_S_REMOTE | VFS_S_USETMP;
+    vfs_init_subclass (&fish_subclass, "fish", VFS_REMOTE | VFS_USETMP, "sh");
+    vfs_fish_ops->fill_names = fish_fill_names;
+    vfs_fish_ops->stat = fish_stat;
+    vfs_fish_ops->lstat = fish_lstat;
+    vfs_fish_ops->fstat = fish_fstat;
+    vfs_fish_ops->chmod = fish_chmod;
+    vfs_fish_ops->chown = fish_chown;
+    vfs_fish_ops->utime = fish_utime;
+    vfs_fish_ops->open = fish_open;
+    vfs_fish_ops->symlink = fish_symlink;
+    vfs_fish_ops->link = fish_link;
+    vfs_fish_ops->unlink = fish_unlink;
+    vfs_fish_ops->rename = fish_rename;
+    vfs_fish_ops->mkdir = fish_mkdir;
+    vfs_fish_ops->rmdir = fish_rmdir;
+    vfs_fish_ops->ctl = fish_ctl;
     fish_subclass.archive_same = fish_archive_same;
+    fish_subclass.new_archive = fish_new_archive;
     fish_subclass.open_archive = fish_open_archive;
     fish_subclass.free_archive = fish_free_archive;
+    fish_subclass.fh_new = fish_fh_new;
     fish_subclass.fh_open = fish_fh_open;
-    fish_subclass.fh_free_data = fish_fh_free_data;
     fish_subclass.dir_load = fish_dir_load;
     fish_subclass.file_store = fish_file_store;
     fish_subclass.linear_start = fish_linear_start;
     fish_subclass.linear_read = fish_linear_read;
     fish_subclass.linear_close = fish_linear_close;
-
-    vfs_s_init_class (&vfs_fish_ops, &fish_subclass);
-    vfs_fish_ops.name = "fish";
-    vfs_fish_ops.prefix = "sh";
-    vfs_fish_ops.fill_names = fish_fill_names;
-    vfs_fish_ops.stat = fish_stat;
-    vfs_fish_ops.lstat = fish_lstat;
-    vfs_fish_ops.fstat = fish_fstat;
-    vfs_fish_ops.chmod = fish_chmod;
-    vfs_fish_ops.chown = fish_chown;
-    vfs_fish_ops.utime = fish_utime;
-    vfs_fish_ops.open = fish_open;
-    vfs_fish_ops.symlink = fish_symlink;
-    vfs_fish_ops.link = fish_link;
-    vfs_fish_ops.unlink = fish_unlink;
-    vfs_fish_ops.rename = fish_rename;
-    vfs_fish_ops.mkdir = fish_mkdir;
-    vfs_fish_ops.rmdir = fish_rmdir;
-    vfs_fish_ops.ctl = fish_ctl;
-    vfs_register_class (&vfs_fish_ops);
+    vfs_register_class (vfs_fish_ops);
 }
 
 /* --------------------------------------------------------------------------------------------- */
