@@ -1,7 +1,7 @@
 /* Virtual File System: SFTP file system.
    The internal functions: dirs
 
-   Copyright (C) 2011-2018
+   Copyright (C) 2011-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -43,7 +43,7 @@
 typedef struct
 {
     LIBSSH2_SFTP_HANDLE *handle;
-    sftpfs_super_data_t *super_data;
+    sftpfs_super_t *super;
 } sftpfs_dir_data_t;
 
 /*** file scope variables ************************************************************************/
@@ -67,7 +67,7 @@ sftpfs_opendir (const vfs_path_t * vpath, GError ** mcerror)
 {
     sftpfs_dir_data_t *sftpfs_dir;
     struct vfs_s_super *super;
-    sftpfs_super_data_t *super_data;
+    sftpfs_super_t *sftpfs_super;
     const vfs_path_element_t *path_element;
     LIBSSH2_SFTP_HANDLE *handle;
 
@@ -78,7 +78,7 @@ sftpfs_opendir (const vfs_path_t * vpath, GError ** mcerror)
     if (vfs_s_get_path (vpath, &super, 0) == NULL)
         return NULL;
 
-    super_data = (sftpfs_super_data_t *) super->data;
+    sftpfs_super = SFTP_SUPER (super);
 
     while (TRUE)
     {
@@ -89,19 +89,19 @@ sftpfs_opendir (const vfs_path_t * vpath, GError ** mcerror)
         fixfname = sftpfs_fix_filename (path_element->path, &fixfname_len);
 
         handle =
-            libssh2_sftp_open_ex (super_data->sftp_session, fixfname, fixfname_len, 0, 0,
+            libssh2_sftp_open_ex (sftpfs_super->sftp_session, fixfname, fixfname_len, 0, 0,
                                   LIBSSH2_SFTP_OPENDIR);
         if (handle != NULL)
             break;
 
-        libssh_errno = libssh2_session_last_errno (super_data->session);
-        if (!sftpfs_waitsocket (super_data, libssh_errno, mcerror))
+        libssh_errno = libssh2_session_last_errno (sftpfs_super->session);
+        if (!sftpfs_waitsocket (sftpfs_super, libssh_errno, mcerror))
             return NULL;
     }
 
     sftpfs_dir = g_new0 (sftpfs_dir_data_t, 1);
     sftpfs_dir->handle = handle;
-    sftpfs_dir->super_data = super_data;
+    sftpfs_dir->super = sftpfs_super;
 
     return (void *) sftpfs_dir;
 }
@@ -132,7 +132,7 @@ sftpfs_readdir (void *data, GError ** mcerror)
         if (rc >= 0)
             break;
 
-        if (!sftpfs_waitsocket (sftpfs_dir->super_data, rc, mcerror))
+        if (!sftpfs_waitsocket (sftpfs_dir->super, rc, mcerror))
             return NULL;
     }
     while (rc == LIBSSH2_ERROR_EAGAIN);
@@ -181,7 +181,7 @@ sftpfs_mkdir (const vfs_path_t * vpath, mode_t mode, GError ** mcerror)
 {
     int res;
     struct vfs_s_super *super;
-    sftpfs_super_data_t *super_data;
+    sftpfs_super_t *sftpfs_super;
     const vfs_path_element_t *path_element;
 
     mc_return_val_if_error (mcerror, -1);
@@ -194,8 +194,8 @@ sftpfs_mkdir (const vfs_path_t * vpath, mode_t mode, GError ** mcerror)
     if (super == NULL)
         return -1;
 
-    super_data = (sftpfs_super_data_t *) super->data;
-    if (super_data->sftp_session == NULL)
+    sftpfs_super = SFTP_SUPER (super);
+    if (sftpfs_super->sftp_session == NULL)
         return -1;
 
     do
@@ -205,11 +205,11 @@ sftpfs_mkdir (const vfs_path_t * vpath, mode_t mode, GError ** mcerror)
 
         fixfname = sftpfs_fix_filename (path_element->path, &fixfname_len);
 
-        res = libssh2_sftp_mkdir_ex (super_data->sftp_session, fixfname, fixfname_len, mode);
+        res = libssh2_sftp_mkdir_ex (sftpfs_super->sftp_session, fixfname, fixfname_len, mode);
         if (res >= 0)
             break;
 
-        if (!sftpfs_waitsocket (super_data, res, mcerror))
+        if (!sftpfs_waitsocket (sftpfs_super, res, mcerror))
             return -1;
     }
     while (res == LIBSSH2_ERROR_EAGAIN);
@@ -231,7 +231,7 @@ sftpfs_rmdir (const vfs_path_t * vpath, GError ** mcerror)
 {
     int res;
     struct vfs_s_super *super;
-    sftpfs_super_data_t *super_data;
+    sftpfs_super_t *sftpfs_super;
     const vfs_path_element_t *path_element;
 
     mc_return_val_if_error (mcerror, -1);
@@ -244,8 +244,8 @@ sftpfs_rmdir (const vfs_path_t * vpath, GError ** mcerror)
     if (super == NULL)
         return -1;
 
-    super_data = (sftpfs_super_data_t *) super->data;
-    if (super_data->sftp_session == NULL)
+    sftpfs_super = SFTP_SUPER (super);
+    if (sftpfs_super->sftp_session == NULL)
         return -1;
 
     do
@@ -255,11 +255,11 @@ sftpfs_rmdir (const vfs_path_t * vpath, GError ** mcerror)
 
         fixfname = sftpfs_fix_filename (path_element->path, &fixfname_len);
 
-        res = libssh2_sftp_rmdir_ex (super_data->sftp_session, fixfname, fixfname_len);
+        res = libssh2_sftp_rmdir_ex (sftpfs_super->sftp_session, fixfname, fixfname_len);
         if (res >= 0)
             break;
 
-        if (!sftpfs_waitsocket (super_data, res, mcerror))
+        if (!sftpfs_waitsocket (sftpfs_super, res, mcerror))
             return -1;
     }
     while (res == LIBSSH2_ERROR_EAGAIN);
