@@ -2,7 +2,7 @@
    Internal file viewer for the Midnight Commander
    Callback function for some actions (hotkeys, menu)
 
-   Copyright (C) 1994-2018
+   Copyright (C) 1994-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -394,17 +394,25 @@ mcview_execute_cmd (WView * view, long command)
             mc_event_raise (MCEVENT_GROUP_CORE, "help", &event_data);
         }
         break;
-    case CK_WrapMode:
-        /* Toggle between wrapped and unwrapped view */
-        mcview_toggle_wrap_mode (view);
+    case CK_HexMode:
+        /* Toggle between hex view and text view */
+        mcview_toggle_hex_mode (view);
         break;
     case CK_HexEditMode:
         /* Toggle between hexview and hexedit mode */
         mcview_toggle_hexedit_mode (view);
         break;
-    case CK_HexMode:
-        /* Toggle between hex view and text view */
-        mcview_toggle_hex_mode (view);
+    case CK_ToggleNavigation:
+        view->hexview_in_text = !view->hexview_in_text;
+        view->dirty++;
+        break;
+    case CK_LeftQuick:
+        if (!view->mode_flags.hex)
+            mcview_move_left (view, 10);
+        break;
+    case CK_RightQuick:
+        if (!view->mode_flags.hex)
+            mcview_move_right (view, 10);
         break;
     case CK_Goto:
         {
@@ -428,24 +436,45 @@ mcview_execute_cmd (WView * view, long command)
     case CK_Search:
         mcview_search (view, TRUE);
         break;
+    case CK_SearchContinue:
+        mcview_continue_search_cmd (view);
+        break;
     case CK_SearchForward:
         mcview_search_options.backwards = FALSE;
         mcview_search (view, TRUE);
         break;
+    case CK_SearchForwardContinue:
+        mcview_search_options.backwards = FALSE;
+        mcview_continue_search_cmd (view);
+        break;
     case CK_SearchBackward:
         mcview_search_options.backwards = TRUE;
         mcview_search (view, TRUE);
         break;
+    case CK_SearchBackwardContinue:
+        mcview_search_options.backwards = TRUE;
+        mcview_continue_search_cmd (view);
+        break;
+    case CK_SearchOppositeContinue:
+        {
+            gboolean direction;
+
+            direction = mcview_search_options.backwards;
+            mcview_search_options.backwards = !direction;
+            mcview_continue_search_cmd (view);
+            mcview_search_options.backwards = direction;
+        }
+        break;
+    case CK_WrapMode:
+        /* Toggle between wrapped and unwrapped view */
+        mcview_toggle_wrap_mode (view);
+        break;
     case CK_MagicMode:
         mcview_toggle_magic_mode (view);
         break;
     case CK_NroffMode:
         mcview_toggle_nroff_mode (view);
         break;
-    case CK_ToggleNavigation:
-        view->hexview_in_text = !view->hexview_in_text;
-        view->dirty++;
-        break;
     case CK_Home:
         mcview_moveto_bol (view);
         break;
@@ -458,28 +487,6 @@ mcview_execute_cmd (WView * view, long command)
     case CK_Right:
         mcview_move_right (view, 1);
         break;
-    case CK_LeftQuick:
-        if (!view->mode_flags.hex)
-            mcview_move_left (view, 10);
-        break;
-    case CK_RightQuick:
-        if (!view->mode_flags.hex)
-            mcview_move_right (view, 10);
-        break;
-    case CK_SearchContinue:
-        mcview_continue_search_cmd (view);
-        break;
-    case CK_SearchForwardContinue:
-        mcview_search_options.backwards = FALSE;
-        mcview_continue_search_cmd (view);
-        break;
-    case CK_SearchBackwardContinue:
-        mcview_search_options.backwards = TRUE;
-        mcview_continue_search_cmd (view);
-        break;
-    case CK_Ruler:
-        mcview_display_toggle_ruler (view);
-        break;
     case CK_Up:
         mcview_move_up (view, 1);
         break;
@@ -507,8 +514,8 @@ mcview_execute_cmd (WView * view, long command)
     case CK_Shell:
         view_other_cmd ();
         break;
-    case CK_BookmarkGoto:
-        view->marks[view->marker] = view->dpy_start;
+    case CK_Ruler:
+        mcview_display_toggle_ruler (view);
         break;
     case CK_Bookmark:
         view->dpy_start = view->marks[view->marker];
@@ -516,6 +523,9 @@ mcview_execute_cmd (WView * view, long command)
         view->dpy_wrap_dirty = TRUE;
         view->dirty++;
         break;
+    case CK_BookmarkGoto:
+        view->marks[view->marker] = view->dpy_start;
+        break;
 #ifdef HAVE_CHARSET
     case CK_SelectCodepage:
         mcview_select_encoding (view);
@@ -701,11 +711,11 @@ mcview_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *
              * panel mode from qick view to another one (by pressing C-x q), the following
              * occurs:
              *   view hook is deleted from select_file_hook list via following call chain:
-             *      set_display_type (view_listing) -> widget_replace () ->
+             *      create_panel (view_listing) -> widget_replace () ->
              *      send_message (MSG_DESTROY) -> mcview_callback (MSG_DESTROY) ->
              *      delete_hook (&select_file_hook);
              *   @view object is free'd:
-             *      set_display_type (view_listing) -> g_free (old_widget);
+             *      create_panel (view_listing) -> g_free (old_widget);
              *   but @view still is in idle_hook list and tried to be executed:
              *      frontend_dlg_run () -> execute_hooks (idle_hook).
              * Thus here we have access to free'd @view object. To prevent this, remove view hook
