@@ -1,7 +1,7 @@
 /*
    lib/vfs - manipulate with current directory
 
-   Copyright (C) 2011-2017
+   Copyright (C) 2011-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -27,14 +27,16 @@
 
 #include "tests/mctest.h"
 
+#include <string.h>             /* memset() */
+
 #include "lib/global.h"
 #include "lib/strutil.h"
 #include "lib/vfs/xdirentry.h"
 
 #include "src/vfs/local/local.c"
 
-static struct vfs_s_subclass test_subclass;
-static struct vfs_class vfs_test_ops;
+static struct vfs_s_subclass vfs_test_subclass;
+static struct vfs_class *vfs_test_ops = VFS_CLASS (&vfs_test_subclass);
 
 /* --------------------------------------------------------------------------------------------- */
 
@@ -56,15 +58,12 @@ setup (void)
     str_init_strings (NULL);
 
     vfs_init ();
-    init_localfs ();
+    vfs_init_localfs ();
     vfs_setup_work_dir ();
 
-    vfs_s_init_class (&vfs_test_ops, &test_subclass);
-
-    vfs_test_ops.name = "testfs";
-    vfs_test_ops.prefix = "test";
-    vfs_test_ops.chdir = test_chdir;
-
+    memset (&vfs_test_subclass, 0, sizeof (vfs_test_subclass));
+    vfs_init_class (vfs_test_ops, "testfs", VFS_UNKNOWN, "test");
+    vfs_test_ops->chdir = test_chdir;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -85,8 +84,7 @@ static const struct test_cd_ds
 {
     const char *input_initial_path;
     const char *input_cd_path;
-    const vfs_class_flags_t input_class_flags;
-    const vfs_subclass_flags_t input_subclass_flags;
+    const vfs_flags_t input_class_flags;
 
     const char *expected_cd_path;
 } test_cd_ds[] =
@@ -94,64 +92,55 @@ static const struct test_cd_ds
     { /* 0. */
         "/",
         "/dev/some.file/test://",
-        VFSF_NOLINKS,
-        0,
+        VFS_NOLINKS,
         "/dev/some.file/test://"
     },
     { /* 1. */
         "/",
         "/dev/some.file/test://bla-bla",
-        VFSF_NOLINKS,
-        0,
+        VFS_NOLINKS,
         "/dev/some.file/test://bla-bla"
     },
     { /* 2. */
         "/dev/some.file/test://bla-bla",
         "..",
-        VFSF_NOLINKS,
-        0,
+        VFS_NOLINKS,
         "/dev/some.file/test://"
     },
     { /* 3. */
         "/dev/some.file/test://",
         "..",
-        VFSF_NOLINKS,
-        0,
+        VFS_NOLINKS,
         "/dev"
     },
     { /* 4. */
         "/dev",
         "..",
-        VFSF_NOLINKS,
-        0,
+        VFS_NOLINKS,
         "/"
     },
     { /* 5. */
         "/",
         "..",
-        VFSF_NOLINKS,
-        0,
+        VFS_NOLINKS,
         "/"
     },
     { /* 6. */
         "/",
         "/test://user:pass@host.net/path",
-        VFSF_NOLINKS,
-        VFS_S_REMOTE,
+        VFS_NOLINKS | VFS_REMOTE,
         "/test://user:pass@host.net/path"
     },
     { /* 7. */
         "/test://user:pass@host.net/path",
         "..",
-        VFSF_NOLINKS,
-        VFS_S_REMOTE,
+        VFS_NOLINKS | VFS_REMOTE,
         "/test://user:pass@host.net/"
     },
     { /* 8. */
         "/test://user:pass@host.net/",
         "..",
-        VFSF_NOLINKS,
-        VFS_S_REMOTE,
+        VFS_NOLINKS | VFS_REMOTE,
         "/"
     },
 };
@@ -165,10 +154,9 @@ START_PARAMETRIZED_TEST (test_cd, test_cd_ds)
     /* given */
     vfs_path_t *vpath;
 
-    vfs_test_ops.flags = data->input_class_flags;
-    test_subclass.flags = data->input_subclass_flags;
+    vfs_test_ops->flags = data->input_class_flags;
+    vfs_register_class (vfs_test_ops);
 
-    vfs_register_class (&vfs_test_ops);
     vfs_set_raw_current_dir (vfs_path_from_str (data->input_initial_path));
 
     vpath = vfs_path_from_str (data->input_cd_path);
@@ -185,6 +173,8 @@ START_PARAMETRIZED_TEST (test_cd, test_cd_ds)
         g_free (actual_cd_path);
     }
     vfs_path_free (vpath);
+
+    vfs_unregister_class (vfs_test_ops);
 }
 /* *INDENT-OFF* */
 END_PARAMETRIZED_TEST
@@ -196,11 +186,17 @@ int
 main (void)
 {
     int number_failed;
+    char *cwd;
 
     Suite *s = suite_create (TEST_SUITE_NAME);
     TCase *tc_core = tcase_create ("Core");
     SRunner *sr;
 
+    /* writable directory where check creates temporary files */
+    cwd = g_get_current_dir ();
+    g_setenv ("TEMP", cwd, TRUE);
+    g_free (cwd);
+
     tcase_add_checked_fixture (tc_core, setup, teardown);
 
     /* Add new tests here: *************** */
