@@ -1,7 +1,7 @@
 /*
    lib/vfs - test vfs_s_get_path() function
 
-   Copyright (C) 2011-2015
+   Copyright (C) 2011-2019
    Free Software Foundation, Inc.
 
    Written by:
@@ -38,7 +38,9 @@
 #define ETALON_VFS_URL_NAME "test2://user:pass@host.net"
 
 struct vfs_s_subclass test_subclass1, test_subclass2, test_subclass3;
-struct vfs_class vfs_test_ops1, vfs_test_ops2, vfs_test_ops3;
+static struct vfs_class *vfs_test_ops1 = VFS_CLASS (&test_subclass1);
+static struct vfs_class *vfs_test_ops2 = VFS_CLASS (&test_subclass2);
+static struct vfs_class *vfs_test_ops3 = VFS_CLASS (&test_subclass3);
 
 /* --------------------------------------------------------------------------------------------- */
 
@@ -51,7 +53,6 @@ test1_mock_open_archive (struct vfs_s_super *super, const vfs_path_t * vpath,
     mctest_assert_str_eq (vfs_path_as_str (vpath), "/" ETALON_VFS_URL_NAME ARCH_NAME);
 
     super->name = g_strdup (vfs_path_as_str (vpath));
-    super->data = g_new (char *, 1);
     root = vfs_s_new_inode (vpath_element->class, super, NULL);
     super->root = root;
     return 0;
@@ -81,34 +82,23 @@ test1_mock_archive_same (const vfs_path_element_t * vpath_element, struct vfs_s_
 static void
 setup (void)
 {
-
     str_init_strings (NULL);
 
     vfs_init ();
-    init_localfs ();
+    vfs_init_localfs ();
     vfs_setup_work_dir ();
 
-
-    test_subclass1.flags = VFS_S_REMOTE;
-    vfs_s_init_class (&vfs_test_ops1, &test_subclass1);
-
-    vfs_test_ops1.name = "testfs1";
-    vfs_test_ops1.flags = VFSF_NOLINKS;
-    vfs_test_ops1.prefix = "test1:";
-    vfs_register_class (&vfs_test_ops1);
+    vfs_init_subclass (&test_subclass1, "testfs1", VFS_NOLINKS | VFS_REMOTE, "test1");
     test_subclass1.open_archive = test1_mock_open_archive;
     test_subclass1.archive_same = test1_mock_archive_same;
     test_subclass1.archive_check = NULL;
+    vfs_register_class (vfs_test_ops1);
 
-    vfs_s_init_class (&vfs_test_ops2, &test_subclass2);
-    vfs_test_ops2.name = "testfs2";
-    vfs_test_ops2.prefix = "test2:";
-    vfs_register_class (&vfs_test_ops2);
+    vfs_init_subclass (&test_subclass2, "testfs2", VFS_UNKNOWN, "test2");
+    vfs_register_class (vfs_test_ops2);
 
-    vfs_s_init_class (&vfs_test_ops3, &test_subclass3);
-    vfs_test_ops3.name = "testfs3";
-    vfs_test_ops3.prefix = "test3:";
-    vfs_register_class (&vfs_test_ops3);
+    vfs_init_subclass (&test_subclass3, "testfs3", VFS_UNKNOWN, "test3");
+    vfs_register_class (vfs_test_ops3);
 }
 
 /* --------------------------------------------------------------------------------------------- */
